/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : ${file_name} 154 ${date}  rbarraza 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.filetransferbc.listeners;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;

import java.io.File;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.jbi.JBIException;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.RobustInOnly;
import javax.xml.transform.Source;

import org.easymock.classextension.EasyMock;

import org.objectweb.petals.binding.filetransferbc.FileTransferBCException;
import org.objectweb.petals.binding.filetransferbc.mock.MockDeliveryChannel;
import org.objectweb.petals.binding.filetransferbc.mock.MockFileTransferBCJBIListener;
import org.objectweb.petals.binding.filetransferbc.mock.MockMessageExchange;
import org.objectweb.petals.binding.filetransferbc.mock.MockServiceEndpoint;
import org.objectweb.petals.component.common.handleMessagesExchange.HandleResponses;
import org.objectweb.petals.component.common.xmlmanipulation.SourceHelper;

import junit.framework.TestCase;

/**
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author Rocio BARRAZA - eBMWebsourcing
 * 
 */
public class FileTransferBCJBIListenerTest extends TestCase {

    private String baseDir;

    private Logger logger;

    private String workDir;

    @Override
    public void setUp() throws Exception {

        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        workDir = baseDir + "src" + File.separator + "test-data"
            + File.separator;
        this.logger = EasyMock.createMock(Logger.class);
        super.setUp();
    }

    public void test1ProcessRobustInOnlyMessages() {

        Method method1 = null;

        try {

            method1 = FileTransferBCJBIListener.class.getDeclaredMethod(
                "handleInMessage", new Class[] {MessageExchange.class});

        } catch (SecurityException e1) {
            e1.printStackTrace();
        } catch (NoSuchMethodException e1) {
            e1.printStackTrace();
        }
        Method[] mockedMethods = new Method[] {method1};

        FileTransferBCJBIListener jbilistener = createMock(
            FileTransferBCJBIListener.class, mockedMethods);

        /*
         * Create messageExchange to be handle by processInOnlyMessages function
         */
        MockMessageExchange msgExch = new MockMessageExchange();
        try {
            expect(jbilistener.handleInMessage(msgExch)).andReturn(true);
            replay(jbilistener);

            /*
             * Test case 1: all is right
             */
            jbilistener.processRobustInOnlyMessages(msgExch);
        } catch (FileTransferBCException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }

        verify(jbilistener);

    }

    /**
     * Test 2: processRobustInOnlyMessages() handleInMessage rise exception
     */
    public void test2ProcessRobustInOnlyMessages() {

        FileTransferBCJBIListener jbilistener = new FileTransferBCJBIListener(
            null, this.logger, null, null) {
            public boolean handleInMessage(MessageExchange msg)
                throws FileTransferBCException {

                throw new FileTransferBCException("Error");
            }
        };

        /*
         * Create messageExchange to be handle by processInOnlyMessages function
         */
        MockMessageExchange msgExch = new MockMessageExchange();

        try {
            jbilistener.processRobustInOnlyMessages(msgExch);

        } catch (Exception e) {
            e.printStackTrace();
        }
        assertTrue(msgExch.isFault());
    }

    public void testHandleInMessage() {

        MockFileTransferBCJBIListener jbilistener = new MockFileTransferBCJBIListener(
            null, this.logger, null, workDir);

        /*
         * Create messageExchange to be handle by handleInMessage function
         */
        Source content = SourceHelper.createContentSource("content");

        MockServiceEndpoint endpoint = createMock(MockServiceEndpoint.class);
        NormalizedMessage nm = createMock(NormalizedMessage.class);
        Set<String> attachs = new HashSet<String>();
        attachs.add("ID");

        MessageExchange msg = createMock(MessageExchange.class);
        try {
            expect(msg.getMessage("IN")).andReturn(nm);
            expect(msg.getEndpoint()).andReturn(endpoint);
            expect(nm.getAttachmentNames()).andReturn(attachs);
            expect(nm.getContent()).andReturn(content);
            expect(endpoint.getEndpointName()).andReturn("endpointName");

            replay(endpoint);
            replay(msg);
            replay(nm);
            // Function to test

            jbilistener.handleInMessage(msg);
        } catch (FileTransferBCException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }

        verify(endpoint);
        verify(msg);
        verify(nm);
        assertTrue(jbilistener.isWriteAttachments());
        assertTrue(jbilistener.isWriteFile());
    }

    public void testProcess() {

        MockDeliveryChannel mockDel = new MockDeliveryChannel();

        FileTransferBCJBIListener jbilistener = new FileTransferBCJBIListener(
            mockDel, this.logger, null, null) {
            public boolean writeFile(String body, String subDir, String fileName) {
                return true;
            }

            protected void processInOnlyMessages(MessageExchange msg) {
            }

            protected void processRobustInOnlyMessages(MessageExchange msg) {
            }
        };

        jbilistener.responseDirName = workDir;

        /*
         * Test case1: message InOnly
         */

        InOnly inOnly = null;
        try {
            inOnly = mockDel.createExchangeFactory().createInOnlyExchange();
        } catch (MessagingException e1) {
            e1.printStackTrace();
        }

        try {
            jbilistener.process(inOnly);

        } catch (Exception e) {
            e.printStackTrace();
        }

        /*
         * Test case2: message InOnly
         */
        RobustInOnly robust = null;
        try {
            robust = mockDel.createExchangeFactory()
                .createRobustInOnlyExchange();
        } catch (MessagingException e1) {
            e1.printStackTrace();
        }

        try {
            jbilistener.process(robust);

        } catch (Exception e) {
            e.printStackTrace();
        }

        /*
         * Test case3: message InOnly
         */
        InOut inOut = null;
        try {
            inOut = mockDel.createExchangeFactory().createInOutExchange();
        } catch (MessagingException e1) {
            e1.printStackTrace();
        }

        try {
            jbilistener.process(inOut);

        } catch (Exception e) {
            e.printStackTrace();
        }

        /*
         * Test case4: message status is ERROR
         */

        try {

            MockServiceEndpoint endpoint = createMock(MockServiceEndpoint.class);
            HandleResponses.faultMessageExchange(inOnly,
                new FileTransferBCException("Error"), "code");
            inOnly.setEndpoint(endpoint);
        } catch (JBIException e) {
            e.printStackTrace();
        }

        try {
            jbilistener.process(inOnly);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void testProcessInOnlyMessages() {

        Method method1 = null;

        try {

            method1 = FileTransferBCJBIListener.class.getDeclaredMethod(
                "handleInMessage", new Class[] {MessageExchange.class});

        } catch (SecurityException e1) {
            e1.printStackTrace();
        } catch (NoSuchMethodException e1) {
            e1.printStackTrace();
        }
        Method[] mockedMethods = new Method[] {method1};

        FileTransferBCJBIListener jbilistener = createMock(
            FileTransferBCJBIListener.class, mockedMethods);

        /*
         * Create messageExchange to be handle by processInOnlyMessages function
         */

        MockMessageExchange msgExch = new MockMessageExchange();
        NormalizedMessage nm = null;
        try {
            nm = msgExch.createMessage();
        } catch (MessagingException e1) {
            e1.printStackTrace();
        }

        msgExch.setMessage(nm, "IN");
        try {
            expect(jbilistener.handleInMessage(msgExch)).andReturn(true);
            replay(jbilistener);

            jbilistener.processInOnlyMessages(msgExch);
        } catch (FileTransferBCException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }

        verify(jbilistener);
    }

    public void testWriteAttachments() {

        FileTransferBCJBIListener jbiList = new FileTransferBCJBIListener(null,
            logger, null, workDir);

        // Get attachments
        File[] files = (new File(workDir + "attachments")).listFiles();
        ArrayList<DataHandler> attachs = new ArrayList<DataHandler>();
        for (File file2 : files) {
            if (!(".svn").equals(file2.getName())) {
                DataHandler dhFile = new DataHandler(new FileDataSource(file2));
                attachs.add(dhFile);
            }
        }
        NormalizedMessage nm = createMock(NormalizedMessage.class);

        Set<String> ids = new HashSet<String>();
        ids.add("ID1");
        ids.add("ID2");

        expect(nm.getAttachment("ID1")).andReturn(attachs.get(0));
        expect(nm.getAttachment("ID2")).andReturn(attachs.get(1));
        replay(nm);

        try {
            jbiList.writeAttachments(ids, nm, "subDir");
        } catch (FileTransferBCException e) {
            e.printStackTrace();
        }
        File[] writenFile = (new File(workDir + "download" + File.separator
            + "subDir")).listFiles();

        DataHandler toRemove = null;
        for (File file : writenFile) {
            for (DataHandler handler : attachs) {
                if (handler.getName().equals(file.getName())) {
                    toRemove = handler;
                }
            }
            attachs.remove(toRemove);
            file.delete();
        }

        verify(nm);
        assertTrue(attachs.isEmpty());

    }

    public void testWriteFile() {

        FileTransferBCJBIListener jbiList = new FileTransferBCJBIListener(null,
            logger, null, workDir);
        String subDir = "subDir";
        String fileName = "fileName";
        try {
            jbiList.writeFile("body", subDir, fileName);
        } catch (FileTransferBCException e) {
            e.printStackTrace();
        }

        File[] writenFile = (new File(workDir + "download" + File.separator
            + subDir)).listFiles();

        boolean written = false;
        for (File file : writenFile) {
            if (fileName.equals(file.getName())) {
                written = true;
            }
            file.delete();
        }
        assertTrue(written);
    }
}
