/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: QueueServiceUnitHandler.java 601 2006-06-13 12:53:50Z alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.jms.su;

import java.io.File;
import java.io.FileFilter;
import java.io.FileNotFoundException;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.management.DeploymentException;
import javax.jbi.servicedesc.ServiceEndpoint;

import org.objectweb.petals.component.common.serviceunitmanager.handler.PetalsServiceUnitHandler;
import org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager;
import org.objectweb.petals.component.common.xmlmanipulation.ManagementMessageUtil;
import org.objectweb.petals.component.common.xmlmanipulation.WSDLHelper;
import org.objectweb.petals.component.common.xmlmanipulation.XMLHelper;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import static org.objectweb.petals.component.common.xmlmanipulation.WSDLHelper.createDocumentFromWSDL;

/**
 * This class is used to deploy external Queue or Topic as service
 * 
 * @version $Revision: $ $Date: $
 * @since Petals 1.0
 * @author alouis
 */
public class JmsServiceUnitHandler extends PetalsServiceUnitHandler {

    protected HashMap<String, JmsConnection> jmsConnexions;

    protected HashMap<String, String> suRootPaths;

    protected HashMap<String, Document> descriptions;

    protected HashMap<ServiceEndpoint, JmsConnection> endpointConnections;

    public static final String TYPE = "jms";

    protected WsdlFileFilter wsdlFileFilter = new WsdlFileFilter();

    public JmsServiceUnitHandler(ComponentContext context, Logger logger) {
        this.logger = logger;
        this.context = context;

        jmsConnexions = new HashMap<String, JmsConnection>();
        suRootPaths = new HashMap<String, String>();
        descriptions = new HashMap<String, Document>();
        endpointConnections = new HashMap<ServiceEndpoint, JmsConnection>();
    }

    /**
     * TODO add customized message
     * (non-Javadoc)
     * @see org.objectweb.petals.component.common.serviceunitmanager.handler.PetalsServiceUnitHandler#deploy(java.lang.String, java.lang.String, java.lang.String)
     */
    public String deploy(String serviceUnitName, String serviceUnitType,
        String serviceUnitRootPath) throws DeploymentException {

        String result = null;

        if (jmsConnexions.containsKey(serviceUnitName)) {
            // already bound
            result = ManagementMessageUtil.getComponentTaskResult(context
                .getComponentName(), "deploy",
                ManagementMessageUtil.TASK_RESULT_FAILED);
        } else {
            JmsConnection conn;

            try {
                if (isSupportedTypeServiceUnit(serviceUnitRootPath)) {
                    // supported service unit

                    conn = JmsConnectionFactory.instance().createConnexion(
                        serviceUnitRootPath);

                    jmsConnexions.put(serviceUnitName, conn);
                    suRootPaths.put(serviceUnitName, serviceUnitRootPath);

                    Document wsdlDescription = findWSDLDescription(serviceUnitRootPath);

                    descriptions.put(serviceUnitName, wsdlDescription);

                    logger.log(Level.FINE, "deploy  serviceUnitName  "
                        + serviceUnitName + "  serviceUnitRootPath "
                        + serviceUnitRootPath);

                    result = ManagementMessageUtil.getComponentTaskResult(
                        context.getComponentName(), "deploy",
                        ManagementMessageUtil.TASK_RESULT_SUCCESS);
                } else {
                    // not supported service unit, return null
                }
            } catch (Exception e1) {
                logger.log(Level.SEVERE,
                    "Exception occured during deployment.", e1);

                result = ManagementMessageUtil.getComponentTaskResult(context
                    .getComponentName(), "deploy",
                    ManagementMessageUtil.TASK_RESULT_FAILED);
            }
        }
        return result;
    }

    public void init(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {

        JmsConnection conn = jmsConnexions.get(serviceUnitName);

        if (conn != null) {
            try {
                conn.init();
            } catch (Exception e) {
                throw new DeploymentException("Can not init serviceUnit '"
                    + serviceUnitName + "'.", e);
            }
        } else {
            throw new DeploymentException("Can not init serviceUnit '"
                + serviceUnitName + "'  : not deployed");
        }

    }

    /**
     * Register the Queue or Topic in JBI with a new ServiceEndpoint
     */
    public void start(String serviceUnitName, PetalsServiceUnitManager epHandler)
        throws DeploymentException {
        logger.log(Level.FINE, "start  serviceUnitName  " + serviceUnitName);

        try {
            // activate the endpoint
            String suRootPath = suRootPaths.get(serviceUnitName);
            activateEndpointsFromJBIDescription(epHandler, descriptions
                .get(serviceUnitName), serviceUnitName, suRootPath);

            // associate the jms connection with the endpoint
            ServiceEndpoint endpoint = epHandler
                .getServiceUnitEndpoint(serviceUnitName);
            JmsConnection conn = jmsConnexions.get(serviceUnitName);
            endpointConnections.put(endpoint, conn);
        } catch (Exception ex) {
            logger.log(Level.SEVERE, FAILED_ACTIVATE_ENDPOINT + ex);
            throw new DeploymentException(FAILED_ACTIVATE_ENDPOINT, ex);
        }
    }

    public void stop(String serviceUnitName, PetalsServiceUnitManager epHandler)
        throws DeploymentException {
        logger.log(Level.FINE, "stop  serviceUnitName  " + serviceUnitName);
    }

    /**
     * Deactivate the corresponding service endpoint in the JBI environment
     */
    public void shutDown(String serviceUnitName,
        PetalsServiceUnitManager epHandler) throws DeploymentException {
        logger.log(Level.FINE, "shutdown  serviceUnitName  " + serviceUnitName);

        try {
            ServiceEndpoint endpoint = epHandler
                .getServiceUnitEndpoint(serviceUnitName);

            // deactivate the endpoint
            deactivateEndpointsFromJBIDescription(serviceUnitName, epHandler);

            // remove ep->connection association
            endpointConnections.remove(endpoint);

            //TODO deactivate from SU manager
        } catch (JBIException e) {
            logger.log(Level.SEVERE, FAILED_DEACTIVATE_ENDPOINT + e);
            throw new DeploymentException(FAILED_DEACTIVATE_ENDPOINT, e);
        }
    }

    public String undeploy(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {
        jmsConnexions.remove(serviceUnitName);
        descriptions.remove(serviceUnitName);
        suRootPaths.remove(serviceUnitName);

        return ManagementMessageUtil.getComponentTaskResult(context
            .getComponentName(), "undeploy",
            ManagementMessageUtil.TASK_RESULT_SUCCESS);
    }

    /**
     * Return the JmsConnection associated with the given endpoint
     * @param endpoint
     * @return the connection, null if no association is found
     */
    public JmsConnection getJmsConnection(ServiceEndpoint endpoint) {
        return endpointConnections.get(endpoint);
    }

    /**
     * Checks if this ServiceUnitHandler can deploy this service unit
     * ( in JBI.XML : services/type = jms )
     * @return true if the service unit type is supported or else false
     */
    protected boolean isSupportedTypeServiceUnit(String serviceUnitRootPath) {

        boolean result = false;

        File jbiXmlFile = new File(serviceUnitRootPath + File.separator
            + "META-INF" + File.separator + "jbi.xml");

        Document configDoc = WSDLHelper.createDocumentFromWSDL(jbiXmlFile);

        Node node = XMLHelper.findChild(configDoc, "services", true);
        if (node != null) {
            NodeList nodeList = node.getChildNodes();
            for (int i = 0; i < nodeList.getLength(); i++) {
                Node itm = nodeList.item(i);
                if (itm.getNodeType() == Node.ELEMENT_NODE
                    && itm.getNodeName().equals("type")) {
                    if (TYPE.equalsIgnoreCase(itm.getTextContent())) {
                        result = true;
                    }
                }
            }
        }
        return result;
    }

    /**
     * Search in the <code>serviceUnitRootPath</code> directory
     * a WSDL file, then create a <code>Document</code> object
     * representing it. 
     * @param serviceUnitRootPath directory where a WSDL file can be found
     * @return a <code>Document</code>
     * @throws Exception WSDL file not found, or can not create a Document from it. 
     */
    protected Document findWSDLDescription(String serviceUnitRootPath)
        throws Exception {
        File[] files = new File(serviceUnitRootPath).listFiles(wsdlFileFilter);

        Document wsdl = null;

        if (files == null || files.length <= 0) {
            throw new FileNotFoundException("No WSDL file was found in '"
                + serviceUnitRootPath + "' directory.");
        }
        wsdl = createDocumentFromWSDL(files[0]);

        if (wsdl == null) {
            throw new Exception(
                "Can not create a Document object from the WSDL file.");
        }
        return wsdl;
    }

    class WsdlFileFilter implements FileFilter {
        public boolean accept(File f) {
            return f.getName().toLowerCase().endsWith(".wsdl");
        }
    }
}
