/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id:  SMTPBC.java 154 2006-03-27 15:30:10Z rbarraza $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.smtpbc;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.xml.namespace.QName;

import org.ebmwebsourcing.b2bgw.security.SecurityManager;
import org.ebmwebsourcing.b2bgw.security.beans.CertificatePKCS12;
import org.objectweb.petals.binding.exception.SMTPBCException;
import org.objectweb.petals.binding.smtpbc.listeners.SMTPBCJBIListener;
import org.objectweb.petals.binding.smtpbc.listeners.SMTPBCListener;
import org.objectweb.petals.component.common.logging.ComponentLogger;
import org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager;
import org.objectweb.petals.component.common.xmlmanipulation.XMLHelper;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Node;

/**
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author alouis
 * 
 */
public class SMTPBC implements Component, ComponentLifeCycle {

    protected ComponentContext context;

    protected SMTPBCJBIListener jbiListener;

    protected SMTPBCListener listener;

    protected Logger logger;

    protected PetalsServiceUnitManager suManager;

    private CertificatePKCS12 cert;

    private DeliveryChannel channel;

    public SMTPBC() {
        super();
    }

    public ObjectName getExtensionMBeanName() {
        return null;
    }

    public ComponentLifeCycle getLifeCycle() {
        return this;
    }

    /**
     * 
     * Load all properties to send and to receive message in connectionProps
     * variable
     * 
     * @throws SMTPBCException
     */
    public Properties getProperties() throws SMTPBCException {
        Properties props = new Properties();
        // Load initial properties file
        try {
            props.load(new FileInputStream(context.getInstallRoot()
                + File.separator + "META-INF" + File.separator
                + "connection.properties"));

            cert = SecurityManager.loadPKCS12Certificate(new FileInputStream(
                context.getInstallRoot() + File.separator + "certificate"
                    + File.separator + "smtpbc.p12"), "password");
        } catch (FileNotFoundException e) {
            this.logger.log(Level.WARNING, "File not found " + e.getMessage());
        } catch (IOException e) {
            this.logger.log(Level.WARNING,
                "Impossible to read connection.properties " + e.getMessage());
        }

        return props;
    }

    public Document getServiceDescription(ServiceEndpoint arg0) {

        return suManager.getServiceDescription(arg0);
    }

    public ServiceUnitManager getServiceUnitManager() {
        logger.log(Level.INFO,
            "Component instance asked to provide a ServiceUnitManager.");
        return suManager;
    }

    public void init(ComponentContext context) throws JBIException {
        try {

            this.context = context;
            Logger log = context.getLogger("", null);
            logger = new ComponentLogger(log, log.getName(), log
                .getResourceBundleName(), context.getComponentName());
            this.channel = context.getDeliveryChannel();

            // Create service unit manager
            suManager = createServiceUnitManager();

            // Parameters of connection
            Properties props = getProperties();

            this.jbiListener = createSMTPBCJBIListener(props);
            // Create listener of server mail
            this.listener = createSMTPBCListener(props);

        } catch (Exception e) {
            throw new JBIException(e);
        }
        logger.log(Level.INFO, "init");

    }

    public boolean isExchangeWithConsumerOkay(ServiceEndpoint arg0,
        MessageExchange arg1) {

        return true;
    }

    public boolean isExchangeWithProviderOkay(ServiceEndpoint arg0,
        MessageExchange arg1) {
        return true;
    }

    public ServiceEndpoint resolveEndpointReference(DocumentFragment arg0) {
        ServiceEndpoint result = null;
        if (arg0 != null) {
            String prefix = arg0.getPrefix();
            if (prefix == null) {
                prefix = new String();
            } else {
                prefix += ":";
            }
            Node service = XMLHelper.findChild(arg0, prefix + "service", true);

            if (service != null) {
                String serviceRequested = XMLHelper.getAttributeValue(service,
                    "name");
                if (serviceRequested != null) {
                    result = suManager.findEndpointForService(QName
                        .valueOf(serviceRequested));
                }
            }
        }
        return result;
    }

    public void shutDown() throws JBIException {
        this.logger.log(Level.INFO, "shutDown");
        this.listener = null;
        this.jbiListener = null;
        this.suManager = null;
    }

    public void start() throws JBIException {
        this.logger.log(Level.INFO, "start");

        // Start jbi container listener thread
        Thread jbiListenerThread = new Thread(this.jbiListener);
        jbiListenerThread.start();

        // Start server listener thread
        // Thread.currentThread().setContextClassLoader(
        // this.getClass().getClassLoader());
        Thread listenerThread = new Thread(this.listener);
        listenerThread.start();

    }

    public void stop() throws JBIException {
        this.logger.log(Level.INFO, "stop");
        try {
            // Close store and folder
            this.listener.stopProcess();
        } catch (SMTPBCException e) {
            logger.log(Level.SEVERE, e.getMessage());
        }
        this.jbiListener.stopProcessing();
    }

    protected PetalsServiceUnitManager createServiceUnitManager()
        throws JBIException {
        return new PetalsServiceUnitManager(context, logger);
    }

    protected SMTPBCJBIListener createSMTPBCJBIListener(Properties props) {
        // Create listener of JBI container
        return new SMTPBCJBIListener(this.channel, logger, this.context, props,
            cert);
    }

    protected SMTPBCListener createSMTPBCListener(Properties props) {
        // Create listener of server mail
        return new SMTPBCListener(this.logger, cert, props, this.context,
            this.channel);
    }
}
