/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : ${file_name} 154 ${date}  rbarraza 
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.smtpbc.listeners;

import java.io.File;
import java.io.FileInputStream;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.NormalizedMessage;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.Session;
import javax.mail.URLName;
import javax.mail.internet.MimeMessage;
import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.binding.exception.SMTPBCException;
import org.objectweb.petals.binding.smtpbc.mock.MockDeliveryChannel;
import org.objectweb.petals.binding.smtpbc.mock.MockMessageExchange;
import org.objectweb.petals.binding.smtpbc.mock.MockSenderMail;
import org.objectweb.petals.binding.smtpbc.mock.MockServiceEndpoint;
import org.objectweb.petals.binding.smtpbc.mock.MockStore;
import org.objectweb.petals.component.common.xmlmanipulation.WSDLHelper;
import org.w3c.dom.Document;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;

/**
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author Rocio BARRAZA - eBMWebsourcing
 * 
 */
public class SMTPBCListenerTest extends TestCase {

    DeliveryChannel channel;

    ComponentContext contextFictive;

    SMTPBCListener listener;

    Logger logger;

    private String baseDir;

    private Properties props;

    @Override
    public void setUp() throws Exception {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        channel = new MockDeliveryChannel();
        props = new Properties();
        props.load(new FileInputStream(baseDir
            + "src" + File.separator + "test" + File.separator + "files"
            + File.separator + "META-INF" + File.separator
            + "connection.properties"));

        logger = EasyMock.createMock(Logger.class);
    }

    public void testConnectionToHost() {

        try {

            /**
             * Test 1: Store address is right
             */
            contextFictive = createMock(ComponentContext.class);
            expect(contextFictive.getInstallRoot()).andReturn(baseDir
                + "src" + File.separator + "test" + File.separator + "files")
                .anyTimes();
            replay(contextFictive);
            listener = new SMTPBCListener(logger, null, props, contextFictive,
                channel);
            props.put("host", "192.168.5.2");
            listener.connectionToHost();
        } catch (Exception e) {
        }

        try {

            /**
             * Test 2: Store address is wrong
             */

            props.put("host", "falseHost");
            listener = new SMTPBCListener(logger, null, props, contextFictive,
                channel);
            listener.connectionToHost();
            assertNull(listener.store);
            fail("*** Expected Exception open Store *** \n");
        } catch (Exception e) {
        }
    }

    public void testEraseMail() {
        listener = new SMTPBCListener(logger, null, null, null, null);
        Session session = Session.getDefaultInstance(props, null);
        MockStore store = new MockStore(session, new URLName("url"));
        try {
            listener.folder = store.getDefaultFolder();
        } catch (javax.mail.MessagingException e) {
            e.printStackTrace();
        }
        Message msg = new MimeMessage(session);
        assertTrue(listener.eraseMail(msg));
    }

    public void testFindServiceOperation() {
        listener = new SMTPBCListener(null, null, null, null, null);
        String service = listener.findServiceOperation("service;operation");
        assertEquals(service, "service");
        assertEquals(listener.operation, "operation");

    }

    public void testGetAttachments() {
        NormalizedMessage nm = createMock(NormalizedMessage.class);

        Set<String> ids = new HashSet<String>();
        ids.add("id1");
        File file = new File(baseDir
            + "src" + File.separator + "test" + File.separator + "files"
            + File.separator);
        DataHandler data = new DataHandler(new FileDataSource(file
            + "attach.txt"));

        expect(nm.getAttachmentNames()).andReturn(ids);
        expect(nm.getAttachment("id1")).andReturn(data);

        replay(nm);

        listener = new SMTPBCListener(null, null, null, null, null);
        /*
         * test that getAttachments get the attachments passed in the
         * messageExchange
         */
        Set<DataHandler> set = listener.getAttachments(nm);
        assertEquals(1, set.size());

        for (DataHandler handler : set) {
            assertEquals(data, handler);
        }
        verify(nm);
    }

    public void testHandleFault() {

        String str = baseDir
            + "src" + File.separator + "test" + File.separator + "files";
        contextFictive = createMock(ComponentContext.class);
        MockSenderMail sendermail = new MockSenderMail(null, null, null, null,
            this.logger);

        expect(contextFictive.getInstallRoot()).andReturn(str).anyTimes();
        replay(contextFictive);

        listener = new SMTPBCListener(logger, null, props, contextFictive,
            channel);
        listener.senderMail = sendermail;
        MockMessageExchange msg = new MockMessageExchange();
        try {
            listener.handleFault(msg, "to", "subject");
        } catch (Exception e) {

        }

        verify(contextFictive);
    }

    // SMTPBCListener
    public void testInOnlyMessage() {
        try {

            contextFictive = createMock(ComponentContext.class);

            expect(contextFictive.getInstallRoot()).andReturn(baseDir
                + "src" + File.separator + "test" + File.separator + "files")
                .anyTimes();
            replay(contextFictive);

            channel = new MockDeliveryChannel();
            listener = new SMTPBCListener(logger, null, props, contextFictive,
                channel);
            listener.operation = "operation";
            /*
             * Test 1: perfect case
             */
            try {
                listener.handleInOnlyMessage("test body", QName.valueOf("serviceName"), null);
            } catch (Exception e) {

            }
            /*
             * Test 2: without body
             */
            try {
                listener.handleInOnlyMessage(null, QName.valueOf("serviceName"), null);
            } catch (Exception e) {

            }
            /*
             * Test 3: without operation
             */
            try {
                listener.handleInOnlyMessage("test body", QName.valueOf("serviceName"), null);
            } catch (Exception e) {

            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        verify(contextFictive);
    }

    public void testInOutMessage() {
        contextFictive = createMock(ComponentContext.class);
        MockDeliveryChannel ch = new MockDeliveryChannel();

        // Initialize Mock senderMail class
        MockSenderMail sendermail = new MockSenderMail(null, null, null, null,
            this.logger);
        listener = new SMTPBCListener(logger, null, null, contextFictive, ch);
        listener.senderMail = sendermail;

        // join an attahcment
        String str = baseDir
            + "src" + File.separator + "test" + File.separator + "files"
            + File.separator + "attachments";

        Set<DataHandler> datas = new HashSet<DataHandler>();
        DataHandler handler = null;
        File[] files = new File(str).listFiles();
        for (File file : files) {
            handler = new DataHandler(new FileDataSource(file));
            datas.add(handler);
        }

        try {
            listener.handleInOutMessage("mailCorps", QName.valueOf("service"), "to",
                "subject", datas);
        } catch (SMTPBCException e) {
            e.printStackTrace();
        }

        assertTrue(sendermail.isSend());
    }

    public void testIsInOut() {
        String str = baseDir
            + "src" + File.separator + "test" + File.separator + "files"
            + File.separator + "falseSmtpbc.wsdl";

        contextFictive = createMock(ComponentContext.class);

        File file = new File(str);
        Document wsdl = WSDLHelper.createDocumentFromWSDL(file);

        MockServiceEndpoint endpoint = new MockServiceEndpoint();
        try {
            expect(contextFictive.getEndpointDescriptor(endpoint)).andReturn(
                wsdl).anyTimes();
        } catch (JBIException e) {
            e.printStackTrace();
        }
        replay(contextFictive);

        channel = new MockDeliveryChannel();

        listener = new SMTPBCListener(logger, null, null, contextFictive,
            channel);

        /*
         * Test case 1: operation inOnly
         */
        listener.operation = "inOnlyOperation";
        try {
            assertFalse(listener.isInOut(endpoint));
        } catch (SMTPBCException e) {
        }

        /*
         * Test case 2 operation inOut
         */
        listener.operation = "inOutOperation";
        try {
            assertTrue(listener.isInOut(endpoint));
        } catch (SMTPBCException e) {
        }
    }

    public void testLoadFolder() {
        listener = new SMTPBCListener(logger, null, null, null, null) {
            public void connectionToHost() {
                try {
                    folder = store.getDefaultFolder();
                } catch (javax.mail.MessagingException e) {
                    e.printStackTrace();
                }
            }

            public void manageMessage(Message msg) {

            }
        };

        Session session = Session.getDefaultInstance(new Properties(), null);
        // Get Mock store --
        listener.store = new MockStore(session, new URLName("host"));
        try {
            Folder fold = listener.loadFolder();
            try {
                assertEquals(fold, listener.store.getFolder("folder"));
            } catch (javax.mail.MessagingException e) {
                e.printStackTrace();
            }

            // Test process()
            try {
                // Folder isn't initialized
                listener.process();

                // Folder is initialized
                listener.connectionToHost();
                listener.process();
            } catch (Exception e) {
                e.printStackTrace();
            }

        } catch (SMTPBCException e) {
            e.printStackTrace();
        }
    }

    public void testServiceNotFound() {

        String str = baseDir
            + "src" + File.separator + "test" + File.separator + "files";

        contextFictive = createMock(ComponentContext.class);
        channel = new MockDeliveryChannel();
        MockSenderMail sendermail = new MockSenderMail(null, null, null, null,
            this.logger);

        expect(contextFictive.getInstallRoot()).andReturn(str).anyTimes();
        replay(contextFictive);

        listener = new SMTPBCListener(logger, null, props, contextFictive,
            channel);
        listener.senderMail = sendermail;

        listener.serviceNotfound("from", "service");
    }
}
