/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DeploymentPackage.java 114 2006-03-23 11:33:07Z alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.packaging;

import java.io.IOException;
import java.net.URI;
import java.util.List;
import java.util.zip.ZipFile;

import org.objectweb.petals.PetalsException;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceAssembly;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceUnit;
import org.objectweb.petals.util.ZipUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * JBI Deployment Package. This object should be constructed via
 * JBIPackageBuilder.
 * 
 * @version $Rev: 114 $ $Date: 2006-03-23 12:33:07 +0100 (jeu., 23 mars 2006) $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
class DeploymentPackage extends AbstractJBIPackage {

    /**
     * @param packageZipURI
     * @param descriptor
     * @throws PetalsException
     */
    public DeploymentPackage(URI packageZipURI, JBIDescriptor descriptor,
            Logger logger) {
        super(packageZipURI, descriptor, logger);
    }

    /**
     * Only check the specified artifacts zip declared in service units. The
     * existing of component with specified name and service unit's internal JBI
     * Descriptor validation and <i>services</i> production should occur
     * further during the deployment process.
     * 
     * @see org.objectweb.petals.jbi.management.packaging.JBIPackage#checkContents()
     */
    public void checkContents() throws PetalsException {

        log.start();

        String msg;

        JBIDescriptor descriptor = getDescriptor();
        ServiceAssembly sa = descriptor.getServiceAssembly();

        List<ServiceUnit> serviceUnits = sa.getServiceUnits();

        ZipFile zipArchive = ZipUtil.openZipFile(getArchiveURI());

        try {
            for (ServiceUnit su : serviceUnits) {

                String artifactZip = su.getTargetArtifactsZip();

                if (!ZipUtil.hasEntry(zipArchive, artifactZip)) {
                    // specified target artifact zip not found within archive
                    msg = "The specified target artifact zip in JBI "
                            + "Descriptor's service unit declaration was not "
                            + "found within deployment package Zip archive file.";
                    throw new PetalsException(msg);
                }
            }
        } finally {
            try {
                zipArchive.close();
            } catch (IOException e) {
                throw new PetalsException(
                        "Error while closing the Zip archive file.", e);
            }
        }
        log.end();
    }

}
