/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: AdminService.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.jbi.management.service;

import java.util.Iterator;
import java.util.Map;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.management.lifecycle.ComponentLifeCycle;
import org.objectweb.petals.jbi.management.lifecycle.LifeCycleAbstract;
import org.objectweb.petals.util.InterfaceNamesHelper;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.ParameterCheckHelper;
import org.objectweb.util.monolog.api.Logger;
import org.w3c.dom.Document;

import static org.objectweb.petals.util.InterfaceNamesHelper.LOGGER_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.MONOLOG_FACTORY_NAME;

/**
 * TODO AdminService : check the exceptions catch
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 * @author wjoseph - eBM WebSourcing
 */
public class AdminService implements AdminServiceMBean, BindingController,
    LifeCycleController {

    private static final String[] CLIENT_INTERFACES = new String[] {InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME};

    /**
     * Logger wrapper
     */
    protected LoggingUtil log;

    /**
     * Monolog
     */
    protected Logger logger;

    /**
     * JBI :: JMX Agent Admin
     */
    protected LifeCycleManagerService manager;

    /**
     * Component lifecycle state.
     */
    private String fcState;

    /**
     * Default constructor
     */
    public AdminService() {
        super();
    }

    /**
     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
     *      java.lang.Object)
     */
    public void bindFc(String clientItfName, Object serverItf)
        throws NoSuchInterfaceException, IllegalBindingException,
        IllegalLifeCycleException {

        if (LOGGER_NAME.equals(clientItfName)) {
            logger = (Logger) serverItf;
        } else if (MONOLOG_FACTORY_NAME.equals(clientItfName)) {
            // nothing
        } else if (InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME
            .equals(clientItfName)) {
            manager = (LifeCycleManagerService) serverItf;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

    /**
     * @see javax.jbi.management.AdminServiceMBean#getBindingComponents()
     */
    public ObjectName[] getBindingComponents() {
        log.start();

        ObjectName[] names = new ObjectName[0];

        Map<ObjectName, ComponentLifeCycle> bindings = manager
            .getBindingCompoLifeCycles();

        names = (ObjectName[]) bindings.keySet().toArray(names);

        return names;
    }

    /**
     * @see javax.jbi.management.AdminServiceMBean#getComponentByName(java.lang.String)
     */
    public ObjectName getComponentByName(String name) {
        log.call();
        ParameterCheckHelper.isNullParameterWithLog(name,
            "name must not be null", log);
        ParameterCheckHelper.isEmptyParameterWithLog(name,
            "name must not be empty", log);
        ObjectName on = null;

        // search in bindings

        on = getBindingComponentByName(name);

        if (on == null) {
            // search in engines

            on = getEngineComponentByName(name);
        }
        return on;
    }

    /**
     * @see javax.jbi.management.AdminServiceMBean#getEngineComponents()
     */
    public ObjectName[] getEngineComponents() {
        log.call();

        ObjectName[] names = new ObjectName[0];

        Map<ObjectName, ComponentLifeCycle> engines = manager
            .getEngineCompoLifeCycles();

        names = (ObjectName[]) engines.keySet().toArray(names);

        return names;
    }

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#getFcState()
     */
    public String getFcState() {
        return STARTED.equals(fcState) ? STARTED : STOPPED;
    }

    /**
     * @see org.objectweb.petals.jbi.management.service.AdminServiceMBean#getServiceDescription(String,
     *      ServiceEndpoint)
     */
    public Document getServiceDescription(String componentName,
        ServiceEndpoint endpoint) throws PetalsException {
        log.call();
        Document doc = null;
        ComponentLifeCycle componentLifeCycle = manager
            .getComponentByName(componentName);
        if (componentLifeCycle == null) {
            log.error("No such component " + componentName
                + " in the container");
            throw new PetalsException("No such component " + componentName
                + " in the container");
        }
        doc = componentLifeCycle.getComponent().getServiceDescription(endpoint);
        return doc;
    }

    /**
     * @see javax.jbi.management.AdminServiceMBean#getSystemInfo()
     */
    public String getSystemInfo() {
        return "Petals JBI Container - version: "
            + Package.getPackage("org.objectweb.petals")
                .getImplementationVersion();
    }

    /**
     * @see javax.jbi.management.AdminServiceMBean#getSystemService(java.lang.String)
     */
    public ObjectName getSystemService(String serviceName) {
        log.call();
        ParameterCheckHelper.isNullParameterWithLog(serviceName,
            "serviceName must not be null", log);
        ParameterCheckHelper.isEmptyParameterWithLog(serviceName,
            "serviceName must not be empty", log);
        ObjectName result = null;
        Map systemServices = manager.getSystemServices();

        synchronized (systemServices) {
            for (Iterator iter = systemServices.values().iterator(); iter
                .hasNext()
                && result == null;) {
                ComponentLifeCycle lifeCycle = (ComponentLifeCycle) iter.next();

                if (lifeCycle.getName().equals(serviceName)) {
                    result = lifeCycle.getMBeanName();
                }
            }
        }
        return result;
    }

    /**
     * @see javax.jbi.management.AdminServiceMBean#getSystemServices()
     */
    public ObjectName[] getSystemServices() {
        log.call();

        ObjectName[] names = new ObjectName[0];

        Map<ObjectName, LifeCycleAbstract> services = manager
            .getSystemServices();

        names = (ObjectName[]) services.keySet().toArray(names);

        return names;
    }

    /**
     * @see javax.jbi.management.AdminServiceMBean#isBinding(java.lang.String)
     */
    public boolean isBinding(String componentName) {
        ParameterCheckHelper.isNullParameterWithLog(componentName,
            "componentName must not be null", log);
        ParameterCheckHelper.isEmptyParameterWithLog(componentName,
            "componentName must not be empty", log);
        boolean result = false;
        result = manager.getBindingCompoLifeCycles().containsKey(componentName);
        return result;
    }

    // ----------------------------------------------------------------------
    // Fractal BindingController implementation
    // ----------------------------------------------------------------------

    /**
     * @see javax.jbi.management.AdminServiceMBean#isEngine(java.lang.String)
     */
    public boolean isEngine(String componentName) {
        ParameterCheckHelper.isNullParameterWithLog(componentName,
            "componentName must not be null", log);
        ParameterCheckHelper.isEmptyParameterWithLog(componentName,
            "componentName must not be empty", log);
        boolean result = false;
        result = manager.getEngineCompoLifeCycles().containsKey(componentName);
        return result;
    }

    /**
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.jbi.management.service.AdminServiceMBean#isExchangeWithConsumerOkayForComponent(java.lang.String,
     *      javax.jbi.servicedesc.ServiceEndpoint,
     *      javax.jbi.messaging.MessageExchange)
     */
    public boolean isExchangeWithConsumerOkayForComponent(String componentName,
        ServiceEndpoint internalEndpoint, MessageExchange exchange)
        throws JBIException {
        log.call();
        ComponentLifeCycle componentLifeCycle = manager
            .getComponentByName(componentName);
        if (componentLifeCycle == null) {
            log.error("No such component " + componentName
                + " in the container");
            throw new JBIException("No such component " + componentName
                + " in the container");
        }
        return componentLifeCycle.getComponent().isExchangeWithConsumerOkay(
            internalEndpoint, exchange);
    }

    /**
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.jbi.management.service.AdminServiceMBean#isExchangeWithProviderOkayForComponent(java.lang.String,
     *      javax.jbi.servicedesc.ServiceEndpoint,
     *      javax.jbi.messaging.MessageExchange)
     */
    public boolean isExchangeWithProviderOkayForComponent(String componentName,
        ServiceEndpoint internalEndpoint, MessageExchange exchange)
        throws JBIException {
        log.call();
        ComponentLifeCycle componentLifeCycle = manager
            .getComponentByName(componentName);
        if (componentLifeCycle == null) {
            log.error("No such component " + componentName
                + " in the container");
            throw new JBIException("No such component " + componentName
                + " in the container");
        }
        return componentLifeCycle.getComponent().isExchangeWithProviderOkay(
            internalEndpoint, exchange);
    }

    /**
     * @see org.objectweb.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc() {
        return CLIENT_INTERFACES;
    }

    // ----------------------------------------------------------------------
    // Fractal LifecycleController implementation
    // ----------------------------------------------------------------------

    /**
     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc(String clientItfName)
        throws NoSuchInterfaceException {

        if (InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME
            .equals(clientItfName)) {
            return manager;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
     */
    public void startFc() throws IllegalLifeCycleException {

        log = new LoggingUtil(logger);

        fcState = STARTED;
    }

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
     */
    public void stopFc() throws IllegalLifeCycleException {

        fcState = STOPPED;
    }

    /**
     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
        IllegalBindingException, IllegalLifeCycleException {

        if (LOGGER_NAME.equals(clientItfName)) {
            logger = null;
        } else if (MONOLOG_FACTORY_NAME.equals(clientItfName)) {
            // nothing
        } else if (InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME
            .equals(clientItfName)) {
            manager = null;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

    /**
     * Return the binding component associated with this name.
     * 
     * @param name
     * @return The related objectName, or null
     */
    protected ObjectName getBindingComponentByName(String name) {
        log.call();

        ObjectName result = null;

        Map bindings = manager.getBindingCompoLifeCycles();

        synchronized (bindings) {
            for (Iterator iter = bindings.values().iterator(); iter.hasNext()
                && result == null;) {
                ComponentLifeCycle lifeCycle = (ComponentLifeCycle) iter.next();

                if (lifeCycle.getName().equals(name)) {
                    result = lifeCycle.getMBeanName();
                }
            }
        }
        return result;
    }

    /**
     * Return the engine component associated with this name.
     * 
     * @param name
     * @return The related objectName, or null
     */
    protected ObjectName getEngineComponentByName(String name) {
        log.call();

        ObjectName result = null;

        Map engines = manager.getEngineCompoLifeCycles();

        synchronized (engines) {
            for (Iterator iter = engines.values().iterator(); iter.hasNext()
                && result == null;) {
                ComponentLifeCycle lifeCycle = (ComponentLifeCycle) iter.next();

                if (lifeCycle.getName().equals(name)) {
                    result = lifeCycle.getMBeanName();
                }
            }
        }
        return result;
    }

}
