/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Launcher.java 461 2006-05-24 12:12:45Z ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.system.launch;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.Map.Entry;

import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.ReflectionException;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import javax.naming.Binding;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.LinkRef;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;

import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.adl.FactoryFactory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.jmx.agent.Admin;
import org.objectweb.fractal.jmx.agent.AdminAttributes;
import org.objectweb.fractal.jmx.agent.Introspector;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.management.systemstate.SystemState;
import org.objectweb.petals.system.terminate.ShutdownHook;
import org.objectweb.petals.util.SystemUtil;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * This class is used to start the Petals platform
 * 
 * @author Rafael Marins
 * @version $Rev: 461 $ $Date: 2006-05-24 14:12:45 +0200 (mer., 24 mai 2006) $
 */
public class Launcher {

    /**
     * Petals composite
     */
    protected Component petalsComposite;

    /**
     * 
     */
    public Launcher() {
        super();
    }

    /**
     * @param args
     */
    public static void main(String[] args) {
        try {
            if (args.length > 0 && "stop".equals(args[0])) {
                Launcher launcher = new Launcher();
                launcher.stop(args);
            } else if (args.length > 0 && "JNDIBrowser".equals(args[0])) {
                Launcher launcher = new Launcher();
                launcher.browseJNDI(args);
            } else {
                Launcher launcher = new Launcher();
                launcher.run(args);
            }
        } catch (Throwable e) {
            System.err.println(e.getMessage());
            Component kernelComp = new Launcher().getPetalsComposite();
            if (kernelComp != null) {
                try {
                    LifeCycleController lcc = Fractal
                            .getLifeCycleController(kernelComp);
                    if (lcc.getFcState().equals(LifeCycleController.STARTED)) {
                        lcc.stopFc();
                        System.out.println("Petals is " + lcc.getFcState());
                    }
                } catch (IllegalLifeCycleException e1) {
                    System.err.println("Problem while stopping Petals");
                } catch (NoSuchInterfaceException e2) {
                    System.err.println("Problem while stopping Petals");
                }
            }
            System.err.println("Petals is stopped !");
            System.exit(-1);
        }
    }

    public Component getPetalsComposite() {
        return petalsComposite;
    }

    /**
     * Browse the distributed JNDI directory
     * 
     * Connect to the JNDI directory of this container if no args are specified
     * 
     * Else can connect to any JNDI directory if -host and -port are specified
     * 
     * @param args
     *            argument of the program
     */
    protected void browseJNDI(String[] args) {
        String host = SystemUtil.getHost();
        String port = SystemUtil.getJndiPort();
        String factory = SystemUtil.getJndiFactory();
        int i = 0;
        for (; i < args.length; i++) {
            if ("-host".equals(args[i])) {
                i++;
                host = args[i];
            } else if ("-port".equals(args[i])) {
                i++;
                port = args[i];
            } else if ("-factory".equals(args[i])) {
                i++;
                factory = args[i];
            }
        }
        Properties properties = new Properties();
        properties.setProperty("java.naming.factory.host", host);
        properties.setProperty("java.naming.factory.port", port);
        properties.setProperty("java.naming.factory.initial", factory);
        InitialContext rootContext;
        try {
            rootContext = retrieveContext(properties);
            System.out.println("JNDI directory on " + host + ":" + port);
            System.out.println("/");
            recurseBindings((Context) rootContext, rootContext
                    .listBindings("/"), "\t");
        } catch (NamingException e) {
            System.err.println(e.getMessage());
        }
    }

    /**
     * A utility function allowing to get a component from a content controller
     * 
     * @param cc
     *            ContentController
     * @param name
     *            component name
     * @return
     */
    protected Component getComponentByName(ContentController contentController,
            String name) {
        // The component to be returned
        Component c = null;
        // List content controller subcomponents
        Component[] comps = contentController.getFcSubComponents();
        for (int i = 0; i < comps.length; i++) {
            Component component = comps[i];
            try {
                String componentName = Fractal.getNameController(component)
                        .getFcName();
                if (componentName.equals(name)) {
                    c = component;
                    break;
                }
            } catch (NoSuchInterfaceException e) {
                // Do nothing
            }
        }
        return c;
    }

    /**
     * Initialize Fractal
     * 
     * @throws Exception
     */
    protected void initializeFractal() throws Exception {
        // check required fractal properties
        System.setProperty("fractal.provider",
                "org.objectweb.fractal.julia.Julia");
        System.setProperty("julia.loader",
                "org.objectweb.fractal.julia.loader.DynamicLoader");
        System.setProperty("julia.config", "julia.cfg");

        Map<String, Object> context = new HashMap<String, Object>();
        // Create Fractal ADL factories
        Factory factory = FactoryFactory.getFactory(
                FactoryFactory.FRACTAL_BACKEND, context);
        // System.setSecurityManager(new SecurityManager());
        // Registry.createRegistry(Registry.DEFAULT_PORT, context);
        // Create composite from ADL
        String compositeName = "Petals";
        petalsComposite = (Component) factory.newComponent(compositeName,
                context);

    }

    /**
     * 
     * @throws IOException
     */
    protected void initializeMonolog() throws IOException {
        // Configure Monolog
        URL url1 = this.getClass().getResource("/loggers.properties");
        Properties p1 = new Properties();
        p1.load(url1.openStream());
        String petalsLF = p1.getProperty("handler.petalsLF.output");
        if (petalsLF != null) {
            p1.setProperty("handler.petalsLF.output", new File(SystemUtil
                    .getPetalsInstallDirectory()
                    + File.separator + "logs" + File.separator + petalsLF)
                    .getAbsolutePath());
        }
        String joramLF = p1.getProperty("handler.joramLF.output");
        if (joramLF != null) {
            p1.setProperty("handler.joramLF.output", new File(SystemUtil
                    .getPetalsInstallDirectory()
                    + File.separator + "logs" + File.separator + joramLF)
                    .getAbsolutePath());
        }
        Monolog.initialize();
        Monolog.loadMonologConfiguration(p1);
    }

    /**
     * @see MBeanServerConnection#invoke(ObjectName, String, Object[], String[])
     */
    protected void invokeStopFc(MBeanServerConnection connection,
            ObjectName result) throws InstanceNotFoundException,
        MBeanException, ReflectionException, IOException {
        connection.invoke(result, "stopFc", new Object[0], new String[0]);
    }

    /**
     * Recurse the contexts
     * 
     * @param bindings
     * @throws NamingException
     */
    protected void recurseBindings(Context context,
            NamingEnumeration<Binding> bindings, String tabs)
        throws NamingException {
        while (bindings.hasMoreElements()) {
            Binding binding = (Binding) bindings.next();
            Object object = binding.getObject();
            if (object instanceof Context) {
                System.out.println(tabs + binding.getName());
                try {
                    recurseBindings((Context) object, context
                            .listBindings(binding.getName()), tabs + "\t");
                } catch (Exception e) {
                    recurseNames((Context) object, context.list(binding
                            .getName()), tabs + "\t");
                }
            } else if (object instanceof Map) {
                System.out.println(tabs + "-> " + binding.getName());
                Map<?, ?> map = (Map<?, ?>) object;
                for (Entry<?, ?> entry : map.entrySet()) {
                    System.out.println(tabs + "\t" + "-> (" + entry.getKey()
                            + ") " + entry.getValue());
                }
            } else if (object instanceof LinkRef) {
                LinkRef linkRef = (LinkRef) binding.getObject();
                System.out.println(tabs + "-> " + binding.getName()
                        + " [Link to " + linkRef.getLinkName() + "]");
            } else {
                System.out.println(tabs + "-> " + binding.getObject());
            }
        }
    }

    /**
     * Recurse the contexts
     * 
     * @param bindings
     * @throws NamingException
     */
    protected void recurseNames(Context context,
            NamingEnumeration<NameClassPair> names, String tabs)
        throws NamingException {
        while (names.hasMoreElements()) {
            NameClassPair name = (NameClassPair) names.next();
            System.out.println(tabs + "-> " + name.getName());
        }
    }

    /**
     * Retrieve an initial context
     * 
     * @param properties
     *            properties of the JNDI directory
     * @return Initialcontext
     * @throws NamingException
     *             if an error occurs
     */
    protected InitialContext retrieveContext(Properties properties)
        throws NamingException {
        return new InitialContext(properties);
    }

    /**
     * @see JMXConnectorFactory#connect(JMXServiceURL, Map)
     */
    protected JMXConnector retrieveJMXConnector(JMXServiceURL url,
            Map<String, ?> args) throws IOException {
        return JMXConnectorFactory.connect(url, args);
    }

    protected void startPetals() throws NoSuchInterfaceException,
        IllegalLifeCycleException {
        String componentName = "jndi-server";
        ContentController contentController = Fractal
                .getContentController(petalsComposite);
        Component jndiServer = getComponentByName(contentController,
                componentName);

        LifeCycleController jndiServerLifeCycleController = Fractal
                .getLifeCycleController(jndiServer);
        jndiServerLifeCycleController.startFc();

        // Start Repository Component
        //startRepositoryComponent(contentController);

        // Start the composite
        LifeCycleController lcc = Fractal
                .getLifeCycleController(petalsComposite);
        lcc.startFc();
    }


    protected void run(String[] args) throws Exception {

        // set the compatible SUID for the Qname
        System.setProperty(
                "com.sun.xml.namespace.QName.useCompatibleSerialVersionUID",
                "1.0");

        try {
            initializeMonolog();
        } catch (IOException e) {
            throw new PetalsException(
                    "Monolog can't be correctly initialized !");
        }

        Logger logger = Monolog.initialize().getLogger("root0.Petals.launcher");

        System.setProperty("javax.management.builder.initial",
                "mx4j.server.MX4JMBeanServerBuilder");

        // Initialize Fractal
        initializeFractal();

        // Start Petals
        startPetals();

        // automatically expose all component server interfaces for JMX
        // Admin at startup
        String componentName = "agent-jmx";
        ContentController contentController = Fractal

        .getContentController(petalsComposite);
        Component agent = getComponentByName(contentController, componentName);
        String interfaceName = "admin";
        Admin agentAdmin = (Admin) agent.getFcInterface(interfaceName);
        Introspector.CURRENCY_TIME_LIMIT = "-1";
        agentAdmin.expose();

        LifeCycleController lcc = Fractal
                .getLifeCycleController(petalsComposite);
        interfaceName = "attribute-controller";
        AdminAttributes adminAttributes = (AdminAttributes) agent
                .getFcInterface(interfaceName);
        adminAttributes.getRawMBeanServer().registerMBean(
                Introspector.createMBean(lcc),
                new ObjectName("FC/Petals@" + lcc.hashCode()
                        + ":itf=lifecycle-controller"));

        // recover all entities
        Component jbi = getComponentByName(contentController, "jbi");
        ContentController contentControllerForJBI = Fractal
                .getContentController(jbi);
        Component systemState = getComponentByName(contentControllerForJBI,
                "systemstate-impl");
        SystemState systemStateService = (SystemState) systemState
                .getFcInterface("service");
        systemStateService.recoverAllEntities();

        // Set up the shutdown hook
        Runtime.getRuntime().addShutdownHook(
                new ShutdownHook(lcc, contentController));

        logger.log(BasicLevel.INFO,
                "##### Petals container correctly started #####");

        //waitForEnd();
    }

    /**
     * Stop Petals container (to use when you launch Petals from Eclipse)
     * 
     * @param arguments
     */
    protected void stop(String[] arguments) {
        try {
            String jmxHost = SystemUtil.getHost();
            String jmxPort = SystemUtil.getJmxPort();
            String username = SystemUtil.getJmxUser();
            String password = SystemUtil.getJmxPassword();

            JMXConnector connector = null;
            Properties env = System.getProperties();
            env.put("java.naming.factory.initial",
                    "com.sun.jndi.rmi.registry.RegistryContextFactory");
            int i = 0;
            for (; i < arguments.length; i++) {
                if ("-host".equals(arguments[i])) {
                    i++;
                    jmxHost = arguments[i];
                } else if ("-port".equals(arguments[i])) {
                    i++;
                    jmxPort = arguments[i];
                }
            }

            Map<String, Object> args = new HashMap<String, Object>();
            if (!"".equals(username)) {
                if (password == null) {
                    password = "";
                }
                String[] credentials = new String[] {username, password};
                args.put("jmx.remote.credentials", credentials);
            }
            String urlStr = "service:jmx:rmi:///jndi/rmi://" + jmxHost + ":"
                    + jmxPort + "/management/rmi-jmx-connector";
            JMXServiceURL url = new JMXServiceURL(urlStr);
            connector = retrieveJMXConnector(url, args);
            MBeanServerConnection connection = connector
                    .getMBeanServerConnection();

            ObjectName result = null;
            String pattern = "FC/Petals@*:itf=lifecycle-controller";
            Set objNames;
            ObjectName objName = new ObjectName(pattern);
            objNames = connection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                result = (ObjectName) objNames.iterator().next();
            }
            if (result != null) {
                invokeStopFc(connection, result);
            }
        } catch (Exception e) {
            System.err.println("The container Petals must be disconnected");
        }
    }

    protected void waitForEnd() {
        System.out.println("Press any key to stop Petals");
        try {
            System.in.read();
        } catch (IOException e) {
            e.printStackTrace();
        }
        System.out.println("exiting Petals...");
        System.exit(0);

    }
}
