/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: InstallationPackageTest.java 09:38:58 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.packaging;

import java.io.File;

import junit.framework.TestCase;

import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.management.lifecycle.mock.MockJBIDescriptorBuilder;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

/**
 * Test of the InstallationPackage
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class InstallationPackageTest extends TestCase {

    protected String baseDir;

    public void setUp() {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
    }

    /**
     * The content is valid
     * 
     * @throws PetalsException
     */
    public void testCheckContents() throws PetalsException {
        LoggerFactory loggerFactory = Monolog.initialize();
        Logger logger = loggerFactory.getLogger("com.foo.MyClass");
        InstallationPackage installationPackage = new InstallationPackage(
                new File(baseDir + "src" + File.separator + "test-data"
                    + File.separator + "packages" + File.separator
                    + "installation01.zip").toURI(), MockJBIDescriptorBuilder
                        .buildComponentDescriptorWithClasspath(), logger);

        installationPackage.checkContents();
    }

    /**
     * The content is not valid
     * 
     * @throws PetalsException
     */
    public void testCheckContents1() {
        LoggerFactory loggerFactory = Monolog.initialize();
        Logger logger = loggerFactory.getLogger("com.foo.MyClass");
        InstallationPackage installationPackage = new InstallationPackage(
                new File(baseDir + "src" + File.separator + "test-data"
                    + File.separator + "packages" + File.separator
                    + "installation01.zip").toURI(), MockJBIDescriptorBuilder
                        .buildComponentDescriptorWithBadClasspath(), logger);
        try {
            installationPackage.checkContents();
            fail();
        } catch (Exception e) {

        }
    }

    /**
     * The content is not valid
     * 
     * @throws PetalsException
     */
    public void testCheckContents2() {
        LoggerFactory loggerFactory = Monolog.initialize();
        Logger logger = loggerFactory.getLogger("com.foo.MyClass");
        InstallationPackage installationPackage = new InstallationPackage(
                new File(baseDir + "src" + File.separator + "test-data"
                    + File.separator + "packages" + File.separator
                    + "installation02.zip").toURI(), MockJBIDescriptorBuilder
                        .buildComponentDescriptorWithClasspath(), logger);
        try {
            installationPackage.checkContents();
            fail();
        } catch (Exception e) {

        }
    }

    /**
     * The content is valid
     * 
     * @throws PetalsException
     * 
     * @throws PetalsException
     */
    public void testCheckContents3() throws PetalsException {
        LoggerFactory loggerFactory = Monolog.initialize();
        Logger logger = loggerFactory.getLogger("com.foo.MyClass");
        InstallationPackage installationPackage = new InstallationPackage(
                new File(baseDir + "src" + File.separator + "test-data"
                    + File.separator + "packages" + File.separator
                    + "test-sl-install.zip").toURI(), MockJBIDescriptorBuilder
                        .buildSharedLibDescriptorWithClasspath(), logger);
        installationPackage.checkContents();
    }

}
