/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: EndpointServiceImplTest.java 16:39:45 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.service;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import javax.jbi.component.Component;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;

import junit.framework.TestCase;

import org.objectweb.petals.jbi.management.lifecycle.ComponentLifeCycle;
import org.objectweb.petals.jbi.management.lifecycle.LifeCycleAbstract;
import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;
import org.objectweb.petals.jbi.registry.InternalEndpoint;
import org.objectweb.petals.jbi.routing.mock.ComponentMock;
import org.objectweb.petals.util.InterfaceNamesHelper;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.wrapper.printwriter.LoggerImpl;
import org.w3c.dom.Document;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.reset;
import static org.easymock.classextension.EasyMock.verify;

/**
 * Test of the EndpointServiceImpl
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class AdminServiceTest extends TestCase {

    
    public void testBindFc() {
        /*
         * Test Case 1 The parameter clientItfName is null, an exception will be
         * raised
         */
        AdminService adminService = new AdminService();
        /*
         * Create Test parameters
         */
        String clientItfName = null;
        Object serverItf = null;
        /*
         * Run Test Case 1
         */
        try {
            adminService.bindFc(clientItfName, serverItf);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The parameter clientItfName match the logger interface
         * name The logger attribute of adminService will be set to serverItf
         * value
         */
        serverItf = createMock(LoggerImpl.class);
        clientItfName = InterfaceNamesHelper.LOGGER_NAME;
        /*
         * Run Test Case 2
         */
        try {
            adminService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("logger attribute was not properly set",
                adminService.logger, serverItf);

        /*
         * Test Case 3 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME The manager attribute
         * of adminService will be set to serverItf value
         */
        serverItf = createMock(LifeCycleManagerImpl.class);
        clientItfName = InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME;
        /*
         * Run Test Case 3
         */
        try {
            adminService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                adminService.manager, serverItf);
    }

    public void testGetBindingComponentByName() {
        /*
         * Test Case 1 There is no binding component matching this name
         */
        AdminService adminService = new AdminService();
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        LoggingUtil log = createMock(LoggingUtil.class);
        adminService.log = log;
        ComponentLifeCycle lifecycle = createMock(ComponentLifeCycle.class);
        ObjectName objectName = createMock(ObjectName.class);
        /*
         * Create Test parameters
         */
        Map<ObjectName, ComponentLifeCycle> bindings = new HashMap<ObjectName, ComponentLifeCycle>();
        bindings.put(objectName, lifecycle);
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getBindingCompoLifeCycles())
                .andReturn(bindings);
        expect(lifecycle.getName()).andReturn("bar");
        replay(lifecycle);
        replay(lifeCycleManager);
        adminService.manager = lifeCycleManager;
        ObjectName result = null;
        String name = "foo";
        /*
         * Run Test Case 1
         */
        result = adminService.getBindingComponentByName(name);
        assertNull("The result of invokation should be null", result);
        verify(lifeCycleManager);
        verify(lifecycle);
        /*
         * Test Case 2 There is one binding component matching this name
         */
        reset(lifeCycleManager);
        reset(lifecycle);
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getBindingCompoLifeCycles())
                .andReturn(bindings);
        expect(lifecycle.getName()).andReturn(name);
        expect(lifecycle.getMBeanName()).andReturn(objectName);
        replay(lifecycle);
        replay(lifeCycleManager);
        adminService.manager = lifeCycleManager;
        /*
         * Run Test Case 2
         */
        result = adminService.getBindingComponentByName(name);
        assertEquals("Wrong invokation response", result, objectName);
        verify(lifecycle);
        verify(lifeCycleManager);
    }

    public void testGetBindingComponents() {
        /*
         * Test Case 1 Check if the result respect JBI specifications
         */
        AdminService adminService = new AdminService();
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        ComponentLifeCycle lifecycle = createMock(ComponentLifeCycle.class);
        ObjectName objectName = createMock(ObjectName.class);
        Map<ObjectName, ComponentLifeCycle> bindings = new HashMap<ObjectName, ComponentLifeCycle>();
        bindings.put(objectName, lifecycle);
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getBindingCompoLifeCycles())
                .andReturn(bindings);
        replay(lifeCycleManager);
        adminService.manager = lifeCycleManager;
        LoggingUtil log = createMock(LoggingUtil.class);
        adminService.log = log;
        /*
         * Run Test Case 1
         */
        ObjectName[] result = adminService.getBindingComponents();
        assertNotNull("The result of GetBindingComponents Must be non null",
                result);
        verify(lifeCycleManager);
        assertTrue("Result does not have the right content", result[0]
                .equals(objectName));
    }

    public void testGetComponentByName() {
        /*
         * Create Mocks
         */
        Method method1 = null;
        Method method2 = null;
        try {
            method1 = AdminService.class.getDeclaredMethod(
                    "getBindingComponentByName", new Class[] {String.class});
            method2 = AdminService.class.getDeclaredMethod(
                    "getEngineComponentByName", new Class[] {String.class});

        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        /*
         * Test Case 1 the name parameter is null
         */
        Method[] mockedMethods = new Method[] {method1, method2};
        AdminService adminService = createMock(AdminService.class,
                mockedMethods);
        LifeCycleManagerService lifeCycleManager = new LifeCycleManagerImpl();
        adminService.manager = lifeCycleManager;
        LoggingUtil log = createMock(LoggingUtil.class);
        adminService.log = log;
        String name = null;
        /*
         * Run Test Case 1
         */
        try {
            adminService.getComponentByName(name);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 the name parameter is empty
         */
        name = "";
        /*
         * Run Test Case 2
         */
        try {
            adminService.getComponentByName(name);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }
        /*
         * Test Case 3 No component with this name exists
         */
        name = "foo";
        expect(adminService.getBindingComponentByName(name)).andReturn(null);
        expect(adminService.getEngineComponentByName(name)).andReturn(null);
        replay(adminService);
        /*
         * Run Test Case 3
         */
        ObjectName result = adminService.getComponentByName(name);
        assertNull("The result is incorrect", result);
    }

    public void testGetEngineComponentByName() {
        /*
         * Test Case 1 There is no binding component matching this name
         */
        AdminService adminService = new AdminService();
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        LoggingUtil log = createMock(LoggingUtil.class);
        adminService.log = log;
        ComponentLifeCycle lifecycle = createMock(ComponentLifeCycle.class);
        ObjectName objectName = createMock(ObjectName.class);
        /*
         * Create Test parameters
         */
        Map<ObjectName, ComponentLifeCycle> engines = new HashMap<ObjectName, ComponentLifeCycle>();
        engines.put(objectName, lifecycle);
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getEngineCompoLifeCycles()).andReturn(engines);
        expect(lifecycle.getName()).andReturn("bar");
        replay(lifecycle);
        replay(lifeCycleManager);
        adminService.manager = lifeCycleManager;
        ObjectName result = null;
        String name = "foo";
        /*
         * Run Test Case 1
         */
        result = adminService.getEngineComponentByName(name);
        assertNull("The result of invokation should be null", result);
        verify(lifeCycleManager);
        verify(lifecycle);
        /*
         * Test Case 2 There is one binding component matching this name
         */
        reset(lifeCycleManager);
        reset(lifecycle);
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getEngineCompoLifeCycles()).andReturn(engines);
        expect(lifecycle.getName()).andReturn(name);
        expect(lifecycle.getMBeanName()).andReturn(objectName);
        replay(lifecycle);
        replay(lifeCycleManager);
        adminService.manager = lifeCycleManager;
        /*
         * Run Test Case 2
         */
        result = adminService.getEngineComponentByName(name);
        assertEquals("Wrong invokation response", result, objectName);
        verify(lifecycle);
        verify(lifeCycleManager);
    }

    public void testGetEngineComponents() {
        /*
         * Test Case 1 Check if the result respect JBI specifications
         */
        AdminService adminService = new AdminService();
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        ComponentLifeCycle lifecycle = createMock(ComponentLifeCycle.class);
        ObjectName objectName = createMock(ObjectName.class);
        Map<ObjectName, ComponentLifeCycle> engines = new HashMap<ObjectName, ComponentLifeCycle>();
        engines.put(objectName, lifecycle);
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getEngineCompoLifeCycles()).andReturn(engines);
        replay(lifeCycleManager);
        adminService.manager = lifeCycleManager;
        LoggingUtil log = createMock(LoggingUtil.class);
        adminService.log = log;
        /*
         * Run Test Case 1
         */
        ObjectName[] result = adminService.getEngineComponents();
        assertNotNull("The result of GetEngineComponents Must be non null",
                result);
        verify(lifeCycleManager);
        assertTrue("Result does not have the right content", result[0]
                .equals(objectName));
    }

    public void testGetFcState() {
        /*
         * Test Case 1 We start the adminService, result of getFcState should be
         * AdminService.STARTED
         */
        AdminService adminService = new AdminService();
        try {
            adminService.startFc();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize test");
        }
        String result = null;
        /*
         * Test Case 1
         */
        result = adminService.getFcState();
        assertEquals("Wring returned state", AdminService.STARTED, result);
    }

    public void testGetServiceDescription() {
        /*
         * Test Case 1 No component with this name is registered, an exception
         * is raised
         */
        AdminService adminService = new AdminService();
        LoggingUtil log = createMock(LoggingUtil.class);
        adminService.log = log;
        Document result = null;
        String componentName = "fooComponent";
        ServiceEndpoint endpoint = createMock(InternalEndpoint.class);
        ComponentLifeCycle lifeCycle = createMock(ComponentLifeCycle.class);
        Component component = new ComponentMock();
        /*
         * Run Test Case 1
         */
        try {
            result = adminService
                    .getServiceDescription(componentName, endpoint);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }
        /*
         * Test Case 2 The requested component name exist but does not provide a
         * service description
         */
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        expect(lifeCycle.getComponent()).andReturn(component);
        expect(lifeCycleManager.getComponentByName(componentName)).andReturn(
                lifeCycle);
        replay(lifeCycleManager);
        replay(lifeCycle);
        adminService.manager = lifeCycleManager;
        /*
         * Run Test Case 2
         */
        try {
            result = adminService
                    .getServiceDescription(componentName, endpoint);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertNull("The result should be null", result);
        verify(lifeCycleManager);
        verify(lifeCycle);

        /*
         * Test Case 3 The requested component name exist and its service
         * description is returned
         */
        reset(lifeCycle);
        reset(lifeCycleManager);
        component = createMock(ComponentMock.class);
        Document serviceDesc = createMock(Document.class);
        expect(lifeCycleManager.getComponentByName(componentName)).andReturn(
                lifeCycle);
        expect(lifeCycle.getComponent()).andReturn(component);
        expect(component.getServiceDescription(endpoint))
                .andReturn(serviceDesc);
        replay(lifeCycleManager);
        replay(lifeCycle);
        replay(component);
        /*
         * Run Test Case 3
         */
        try {
            result = adminService
                    .getServiceDescription(componentName, endpoint);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("Wrong result for invokation", result, serviceDesc);
        verify(lifeCycleManager);
        verify(lifeCycle);
        verify(component);
    }

    public void testGetSystemInfo() {
        /*
         * Test Case 1 Checks the result of getSystemInfo respect JBI
         * specifications
         */
        AdminService adminService = new AdminService();
        /*
         * Run Test Case 1
         */
        String result = adminService.getSystemInfo();
        assertNotNull("Result of getSystemInfo must not be null", result);
        assertFalse("Result of getSystemInfo must not be empty", result
                .length() == 0);
    }

    public void testGetSystemService() {
        /*
         * Test Case 1 The serviceName parameter is null, an exception is raised
         */
        AdminService adminService = new AdminService();
        LoggingUtil log = createMock(LoggingUtil.class);
        adminService.log = log;
        /*
         * Run Test Case 1
         */
        /*
         * Create Test parameters
         */
        String serviceName = null;
        try {
            adminService.getSystemService(serviceName);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The serviceName parameter is empty, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        serviceName = "";
        /*
         * Run Test Case 2
         */
        try {
            adminService.getSystemService(serviceName);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 3 The serviceName parameter is empty, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        serviceName = "fooService";
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        Map<ObjectName, LifeCycleAbstract> services = new HashMap<ObjectName, LifeCycleAbstract>();
        ObjectName objectName = createMock(ObjectName.class);
        ComponentLifeCycle lifeCycle = createMock(ComponentLifeCycle.class);
        services.put(objectName, lifeCycle);
        expect(lifeCycleManager.getSystemServices()).andReturn(services);
        expect(lifeCycle.getName()).andReturn(serviceName);
        expect(lifeCycle.getMBeanName()).andReturn(objectName);
        replay(lifeCycleManager);
        replay(lifeCycle);
        adminService.manager = lifeCycleManager;
        ObjectName result = null;
        /*
         * Run Test Case 3
         */
        try {
            result = adminService.getSystemService(serviceName);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        assertEquals("Wrong result for invokation", result, objectName);
        verify(lifeCycle);
        verify(lifeCycleManager);
    }

    public void testGetSystemServices() {
        /*
         * Test Case 1 Checks the result of getSystemServices respect JBI
         * specifications
         */
        AdminService adminService = new AdminService();
        LoggingUtil log = createMock(LoggingUtil.class);
        adminService.log = log;
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        Map<ObjectName, LifeCycleAbstract> services = new HashMap<ObjectName, LifeCycleAbstract>();
        ObjectName objectName = createMock(ObjectName.class);
        ComponentLifeCycle lifeCycle = createMock(ComponentLifeCycle.class);
        services.put(objectName, lifeCycle);
        expect(lifeCycleManager.getSystemServices()).andReturn(services);
        replay(lifeCycleManager);
        adminService.manager = lifeCycleManager;
        /*
         * Run Test Case 1
         */
        ObjectName[] result = adminService.getSystemServices();
        assertNotNull("The result of getSystemServices must not be null",
                result);
        assertTrue("The result is incorrect", result.length == 1);
        verify(lifeCycleManager);
    }

    public void testIsBinding() {
        /*
         * Test Case 1 The componentName parameter is null, an exception is
         * raised
         */
        AdminService adminService = new AdminService();
        LoggingUtil log = createMock(LoggingUtil.class);
        adminService.log = log;
        /*
         * Run Test Case 1
         */
        /*
         * Create Test parameters
         */
        String componentName = null;
        try {
            adminService.isBinding(componentName);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The componentName parameter is empty, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        componentName = "";
        /*
         * Run Test Case 2
         */
        try {
            adminService.isBinding(componentName);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }
        /*
         * Test Case 3 The requested compenent is not registered, false is
         * returned
         */
        /*
         * Create Mocks
         */
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        ComponentLifeCycle lifecycle = createMock(ComponentLifeCycle.class);
        ObjectName objectName = createMock(ObjectName.class);
        /*
         * Create Test parameters
         */
        Map<ObjectName, ComponentLifeCycle> bindings = new HashMap<ObjectName, ComponentLifeCycle>();
        bindings.put(objectName, lifecycle);
        componentName = "fooCompo";
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getBindingCompoLifeCycles())
                .andReturn(bindings);
        replay(lifeCycleManager);
        adminService.manager = lifeCycleManager;
        /*
         * Run Test Case 3
         */
        boolean result = adminService.isBinding(componentName);
        assertFalse("Result of invokation should be false", result);
        verify(lifeCycleManager);
    }

    public void testIsEngine() {
        /*
         * Test Case 1 The componentName parameter is null, an exception is
         * raised
         */
        AdminService adminService = new AdminService();
        LoggingUtil log = createMock(LoggingUtil.class);
        adminService.log = log;
        /*
         * Run Test Case 1
         */
        /*
         * Create Test parameters
         */
        String componentName = null;
        try {
            adminService.isEngine(componentName);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The componentName parameter is empty, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        componentName = "";
        /*
         * Run Test Case 2
         */
        try {
            adminService.isEngine(componentName);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }
        /*
         * Test Case 3 The requested compenent is not registered, false is
         * returned
         */
        /*
         * Create Mocks
         */
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        ComponentLifeCycle lifecycle = createMock(ComponentLifeCycle.class);
        ObjectName objectName = createMock(ObjectName.class);
        /*
         * Create Test parameters
         */
        Map<ObjectName, ComponentLifeCycle> engines = new HashMap<ObjectName, ComponentLifeCycle>();
        engines.put(objectName, lifecycle);
        componentName = "fooCompo";
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getEngineCompoLifeCycles()).andReturn(engines);
        replay(lifeCycleManager);
        adminService.manager = lifeCycleManager;
        /*
         * Run Test Case 3
         */
        boolean result = adminService.isEngine(componentName);
        assertFalse("Result of invokation should be false", result);
        verify(lifeCycleManager);
    }

    public void testIsExchangeWithConsumerOkayForComponent() {
        /*
         * Test Case 1 There is no binding component matching this name
         */
        AdminService adminService = new AdminService();
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        adminService.manager = lifeCycleManager;
        LoggingUtil log = createMock(LoggingUtil.class);
        adminService.log = log;
        /*
         * Create Test parameters
         */
        String componentName = "foo";
        ServiceEndpoint internalEndpoint = createMock(InternalEndpoint.class);
        MessageExchange exchange = createMock(MessageExchangeImpl.class);
        /*
         * Run Test Case 1
         */
        try {
            adminService.isExchangeWithConsumerOkayForComponent(componentName,
                    internalEndpoint, exchange);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 There is one binding component matching this name
         */
        /*
         * Create Test parameters
         */
        Component component = new ComponentMock();
        ComponentLifeCycle lifeCycle = createMock(ComponentLifeCycle.class);
        lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        /*
         * Initialise Mocks
         */
        expect(lifeCycle.getComponent()).andReturn(component);
        expect(lifeCycleManager.getComponentByName(componentName)).andReturn(
                lifeCycle);
        replay(lifeCycleManager);
        replay(lifeCycle);
        adminService.manager = lifeCycleManager;
        boolean result = false;
        /*
         * Run Test Case 2
         */
        try {
            result = adminService.isExchangeWithConsumerOkayForComponent(
                    componentName, internalEndpoint, exchange);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation ");
        }
        assertTrue("the result of invokation should be true", result);
        verify(lifeCycle);
        verify(lifeCycleManager);
    }

    public void testIsExchangeWithProviderOkayForComponent() {
        /*
         * Test Case 1 There is no binding component matching this name
         */
        AdminService adminService = new AdminService();
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        adminService.manager = lifeCycleManager;
        LoggingUtil log = createMock(LoggingUtil.class);
        adminService.log = log;
        /*
         * Create Test parameters
         */
        String componentName = "foo";
        ServiceEndpoint internalEndpoint = createMock(InternalEndpoint.class);
        MessageExchange exchange = createMock(MessageExchangeImpl.class);
        /*
         * Run Test Case 1
         */
        try {
            adminService.isExchangeWithProviderOkayForComponent(componentName,
                    internalEndpoint, exchange);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 There is one binding component matching this name
         */
        /*
         * Create Test parameters
         */
        Component component = new ComponentMock();
        ComponentLifeCycle lifeCycle = createMock(ComponentLifeCycle.class);
        lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        /*
         * Initialise Mocks
         */
        expect(lifeCycle.getComponent()).andReturn(component);
        expect(lifeCycleManager.getComponentByName(componentName)).andReturn(
                lifeCycle);
        replay(lifeCycleManager);
        replay(lifeCycle);
        adminService.manager = lifeCycleManager;
        boolean result = false;
        /*
         * Run Test Case 2
         */
        try {
            result = adminService.isExchangeWithProviderOkayForComponent(
                    componentName, internalEndpoint, exchange);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation ");
        }
        assertTrue("the result of invokation should be true", result);
        verify(lifeCycle);
        verify(lifeCycleManager);
    }

    public void testLookupFc() {
        /*
         * Test Case 1 The requested interface does not belong to adminService
         */
        AdminService adminService = new AdminService();
        /*
         * Create test parameters
         */
        String clientItf = "foo";
        /*
         * Run Test Case 1
         */
        try {
            adminService.lookupFc(clientItf);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The requested interface belongs to adminService
         */
        /*
         * Create test parameters
         */
        clientItf = InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME;
        Object result = null;
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        adminService.manager = lifeCycleManager;
        /*
         * Run Test Case 2
         */
        try {
            result = adminService.lookupFc(clientItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Exception during method invokation");
        }
        assertEquals("Wrong result for invokation", result, lifeCycleManager);
    }

    public void testStartFc() {
        /*
         * Test Case 1
         */
        AdminService adminService = new AdminService();
        /*
         * Run Test Case 1
         */
        try {
            adminService.startFc();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertNotNull("The logger was not initialised", adminService.log);
        assertEquals("The state is not started", adminService.getFcState(),
                AdminService.STARTED);
    }

    public void testStopFc() {
        /*
         * Test Case 1
         */
        AdminService adminService = new AdminService();
        try {
            adminService.startFc();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize test parameters");
        }
        /*
         * Run Test Case 1
         */
        try {
            adminService.stopFc();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertEquals("The state is not stopped", adminService.getFcState(),
                AdminService.STOPPED);
    }

    public void testUnunbindFc() {
        /*
         * Test Case 1 The parameter clientItfName is null, an exception will be
         * raised
         */
        AdminService adminService = new AdminService();
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        adminService.manager = lifeCycleManager;
        LoggingUtil log = createMock(LoggingUtil.class);
        adminService.log = log;
        /*
         * Create Test parameters
         */
        String clientItfName = null;
        /*
         * Run Test Case 1
         */
        try {
            adminService.unbindFc(clientItfName);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The parameter clientItfName match the logger interface
         * name The logger attribute of adminService will be set to null
         */
        clientItfName = InterfaceNamesHelper.LOGGER_NAME;
        /*
         * Run Test Case 2
         */
        try {
            adminService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertNull("logger attribute was not properly set", adminService.logger);

        /*
         * Test Case 3 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME The manager attribute
         * of adminService will be set to null
         */
        clientItfName = InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME;
        /*
         * Run Test Case 3
         */
        try {
            adminService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertNull("manager attribute was not properly set",
                adminService.manager);
    }

    @Override
    protected void setUp() {

    }
}
