/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DeploymentServiceTest.java 16:39:45 wjoseph $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.service;

import java.lang.reflect.Method;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.jbi.component.Component;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.management.DeploymentException;

import junit.framework.TestCase;

import org.objectweb.petals.jbi.management.lifecycle.ComponentLifeCycle;
import org.objectweb.petals.jbi.management.lifecycle.LifeCycleAbstract;
import org.objectweb.petals.jbi.management.lifecycle.ServiceAssemblyLifeCycle;
import org.objectweb.petals.jbi.management.systemstate.SystemState;
import org.objectweb.petals.jbi.management.systemstate.SystemStateImpl;
import org.objectweb.petals.jbi.routing.mock.ComponentMock;
import org.objectweb.petals.repository.RepositoryImpl;
import org.objectweb.petals.repository.RepositoryService;
import org.objectweb.petals.tools.jbicommon.descriptor.Identification;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceAssembly;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceUnit;
import org.objectweb.petals.util.InterfaceNamesHelper;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.wrapper.printwriter.LoggerImpl;

import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.isA;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;

/**
 * Test of the DeploymentService
 * 
 * @author wjoseph - eBMWebsourcing
 */
public class DeploymentServiceTest extends TestCase {

    public void testBindFc() {
        /*
         * Test Case 1 The parameter clientItfName is null, an exception will be
         * raised
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        String clientItfName = null;
        Object serverItf = null;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.bindFc(clientItfName, serverItf);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The parameter clientItfName match the logger interface
         * name The logger attribute of deploymentService will be set to
         * serverItf value
         */
        serverItf = createMock(LoggerImpl.class);
        clientItfName = InterfaceNamesHelper.LOGGER_NAME;
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("logger attribute was not properly set",
                deploymentService.logger, serverItf);

        /*
         * Test Case 3 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME The managerService
         * attribute of deploymentService will be set to serverItf value
         */
        serverItf = createMock(LifeCycleManagerImpl.class);
        clientItfName = InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME;
        /*
         * Run Test Case 3
         */
        try {
            deploymentService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                deploymentService.managerService, serverItf);

        /*
         * Test Case 4 The parameter clientItfName match the
         * InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME The repositorySrv
         * attribute of deploymentService will be set to serverItf value
         */
        serverItf = createMock(RepositoryImpl.class);
        clientItfName = InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME;
        /*
         * Run Test Case 4
         */
        try {
            deploymentService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                deploymentService.repositorySrv, serverItf);

        /*
         * Test Case 5 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_ADMIN_NAMEThe adminSrv attribute of
         * deploymentService will be set to serverItf value
         */
        serverItf = createMock(AdminService.class);
        clientItfName = InterfaceNamesHelper.JBI_ADMIN_NAME;
        /*
         * Run Test Case 5
         */
        try {
            deploymentService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                deploymentService.adminSrv, serverItf);

        /*
         * Test Case 6 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME The recoverySrv attribute
         * of deploymentService will be set to serverItf value
         */
        serverItf = createMock(SystemStateImpl.class);
        clientItfName = InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME;
        /*
         * Run Test Case 6
         */
        try {
            deploymentService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                deploymentService.recoverySrv, serverItf);

        /*
         * Test Case 7 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_ENDPOINTSERVICE_NAME The recoverySrv
         * attribute of endpointService will be set to serverItf value
         */
        serverItf = createMock(EndpointServiceImpl.class);
        clientItfName = InterfaceNamesHelper.JBI_ENDPOINTSERVICE_NAME;
        /*
         * Run Test Case 7
         */
        try {
            deploymentService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                deploymentService.endpointService, serverItf);
    }

    public void testCanDeployToComponent() {
        /*
         * Test Case 1 The parameter componentName is null, an exception will be
         * raised
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        String componentName = null;
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.canDeployToComponent(componentName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }

        /*
         * Test Case 2 The parameter componentName is empty, an exception will
         * be raised
         */
        /*
         * Create Test parameters
         */
        componentName = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.canDeployToComponent(componentName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }

        /*
         * Test Case 3 One component is found with this name and accept
         * deployment
         */
        /*
         * Create Test parameters
         */
        componentName = "fooCompo";
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        ComponentLifeCycle componentLifeCycle = createMock(ComponentLifeCycle.class);
        Component component = new ComponentMock() {
            public javax.jbi.component.ServiceUnitManager getServiceUnitManager() {
                return new ServiceUnitManager() {

                    public String deploy(String serviceUnitName,
                            String serviceUnitRootPath)
                        throws DeploymentException {
                        return null;
                    }

                    public void init(String serviceUnitName,
                            String serviceUnitRootPath)
                        throws DeploymentException {

                    }

                    public void shutDown(String serviceUnitName)
                        throws DeploymentException {

                    }

                    public void start(String serviceUnitName)
                        throws DeploymentException {

                    }

                    public void stop(String serviceUnitName)
                        throws DeploymentException {

                    }

                    public String undeploy(String serviceUnitName,
                            String serviceUnitRootPath)
                        throws DeploymentException {
                        return null;
                    }

                };
            }
        };

        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getComponentByName(componentName)).andReturn(
                componentLifeCycle);
        expect(componentLifeCycle.getCurrentState()).andReturn(
                LifeCycleAbstract.STARTED);
        expect(componentLifeCycle.getComponent()).andReturn(component);
        replay(lifeCycleManager);
        replay(componentLifeCycle);
        deploymentService.managerService = lifeCycleManager;
        boolean result = false;
        /*
         * Run Test Case 3
         */
        try {
            result = deploymentService.canDeployToComponent(componentName);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        assertTrue("Wrong invokation result", result);
    }

    public void testGetComponentsForDeployedServiceAssembly() {
        /*
         * Test Case 1 The parameter serviceAssemblyName is null, an exception
         * will be raised
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        String serviceAssemblyName = null;
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService
                    .getComponentsForDeployedServiceAssembly(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }

        /*
         * Test Case 2 The parameter serviceAssemblyName is empty, an exception
         * will be raised
         */
        /*
         * Create Test parameters
         */
        serviceAssemblyName = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService
                    .getComponentsForDeployedServiceAssembly(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }
        /*
         * Test Case 3 No service assembly for the given name is found, an
         * exception is raised
         */
        /*
         * Create Test parameters
         */
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        deploymentService.managerService = lifeCycleManager;
        serviceAssemblyName = "fooSA";
        /*
         * Run Test Case 3
         */
        try {
            deploymentService
                    .getComponentsForDeployedServiceAssembly(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue(
                    "Wrong exception thrown",
                    e
                            .getMessage()
                            .contains(
                                    "Cannot retrieve service assembly with the specified name : "));
        }

        /*
         * Test Case 4 The service assembly has one service unit
         */
        /*
         * Create Test parameters
         */
        lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        serviceAssemblyName = "fooSA";
        ServiceAssemblyLifeCycle saLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        ServiceAssembly sa = createMock(ServiceAssembly.class);
        List<ServiceUnit> sus = new ArrayList<ServiceUnit>();
        ServiceUnit su = createMock(ServiceUnit.class);
        sus.add(su);
        String componentName = "componentName";
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyByName(serviceAssemblyName))
                .andReturn(saLifeCycle);
        expect(saLifeCycle.getServiceAssembly()).andReturn(sa);
        expect(sa.getServiceUnits()).andReturn(sus);
        expect(su.getTargetComponentName()).andReturn(componentName);
        replay(lifeCycleManager);
        replay(sa);
        replay(saLifeCycle);
        replay(su);
        deploymentService.managerService = lifeCycleManager;
        String[] result = null;
        /*
         * Run Test Case 4
         */
        try {
            result = deploymentService
                    .getComponentsForDeployedServiceAssembly(serviceAssemblyName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertNotNull(
                "The result of getComponentsForDeployedServiceAssembly must not be null",
                result);
        assertEquals("Wrong invokation result", result[0], componentName);
        assertEquals("Wrong invokation result", result.length, 1);
        verify(lifeCycleManager);
        verify(sa);
        verify(saLifeCycle);
        verify(su);
    }

    public void testGetDeployedServiceAssemblies() {
        /*
         * Test Case 1
         * 
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        LifeCycleManagerImpl lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        Map<String, ServiceAssemblyLifeCycle> serviceAssemblies = new HashMap<String, ServiceAssemblyLifeCycle>();
        String saName = "service assembly";
        ServiceAssemblyLifeCycle saLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        serviceAssemblies.put(saName, saLifeCycle);
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyLifeCycles()).andReturn(
                serviceAssemblies);
        replay(lifeCycleManager);
        deploymentService.managerService = lifeCycleManager;
        String[] result = null;
        /*
         * Run Test Case 1
         */
        try {
            result = deploymentService.getDeployedServiceAssemblies();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertNotNull(
                "Result of getDeployedServiceAssemblies must not be null",
                result);
        assertEquals("Wrong invokation result", result[0], saName);
        assertEquals("Wrong invokation result", result.length, 1);
        verify(lifeCycleManager);
    }

    public void testGetDeployedServiceAssembliesForComponent() {
        /*
         * Test Case 1 The parameter componentName is null, an exception is
         * raised
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        String componentName = null;
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService
                    .getDeployedServiceAssembliesForComponent(componentName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }
        /*
         * Test Case 2 The parameter componentName is empty, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        componentName = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService
                    .getDeployedServiceAssembliesForComponent(componentName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }

        /*
         * Test Case 3
         */
        /*
         * Create Test parameters
         */
        componentName = "fooComponent";
        LifeCycleManagerImpl lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        Map<String, ServiceAssemblyLifeCycle> serviceAssemblies = new HashMap<String, ServiceAssemblyLifeCycle>();
        String saName = "service assembly";
        ServiceAssemblyLifeCycle saLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        serviceAssemblies.put(saName, saLifeCycle);
        ServiceAssembly sa = createMock(ServiceAssembly.class);
        List<ServiceUnit> sus = new ArrayList<ServiceUnit>();
        ServiceUnit su = createMock(ServiceUnit.class);
        sus.add(su);
        Identification identification = createMock(Identification.class);
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyLifeCycles()).andReturn(
                serviceAssemblies);
        expect(saLifeCycle.getServiceAssembly()).andReturn(sa).times(2);
        expect(sa.getServiceUnits()).andReturn(sus);
        expect(sa.getIdentification()).andReturn(identification);
        expect(identification.getName()).andReturn(saName);
        expect(su.getTargetComponentName()).andReturn(componentName);
        replay(sa);
        replay(su);
        replay(lifeCycleManager);
        replay(saLifeCycle);
        replay(identification);
        deploymentService.managerService = lifeCycleManager;
        String[] result = null;
        /*
         * Run Test Case 3
         */
        try {
            result = deploymentService
                    .getDeployedServiceAssembliesForComponent(componentName);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        assertNotNull(
                "Result of getDeployedServiceAssembliesForComponent must not be null",
                result);
        assertEquals("Wrong invokation result", result[0], saName);
        verify(saLifeCycle);
        verify(lifeCycleManager);
        verify(sa);
        verify(su);
        verify(identification);
    }

    public void testGetDeployedServiceUnitList() {
        /*
         * Test Case 1 The parameter componentName is null, an exception is
         * raised
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        String componentName = null;
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.getDeployedServiceUnitList(componentName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }
        /*
         * Test Case 2 The parameter componentName is empty, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        componentName = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.getDeployedServiceUnitList(componentName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }

        /*
         * Test Case 3
         */
        /*
         * Create Test parameters
         */
        componentName = "fooComponent";
        LifeCycleManagerImpl lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        Map<String, ServiceAssemblyLifeCycle> serviceAssemblies = new HashMap<String, ServiceAssemblyLifeCycle>();
        String suName = "service unit";
        ServiceAssemblyLifeCycle saLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        serviceAssemblies.put(suName, saLifeCycle);
        ServiceAssembly sa = createMock(ServiceAssembly.class);
        List<ServiceUnit> sus = new ArrayList<ServiceUnit>();
        ServiceUnit su = createMock(ServiceUnit.class);
        sus.add(su);
        Identification identification = createMock(Identification.class);
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyLifeCycles()).andReturn(
                serviceAssemblies);
        expect(saLifeCycle.getServiceAssembly()).andReturn(sa);
        expect(sa.getServiceUnits()).andReturn(sus);
        expect(identification.getName()).andReturn(suName);
        expect(su.getTargetComponentName()).andReturn(componentName);
        expect(su.getIdentification()).andReturn(identification);
        replay(sa);
        replay(su);
        replay(lifeCycleManager);
        replay(saLifeCycle);
        replay(identification);
        deploymentService.managerService = lifeCycleManager;
        String[] result = null;
        /*
         * Run Test Case 3
         */
        try {
            result = deploymentService
                    .getDeployedServiceUnitList(componentName);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        assertNotNull(
                "Result of getDeployedServiceAssembliesForComponent must not be null",
                result);
        assertEquals("Wrong invokation result", result[0], suName);
        verify(saLifeCycle);
        verify(lifeCycleManager);
        verify(sa);
        verify(su);
        verify(identification);
    }

    public void testGetFcState() {
        /*
         * Test Case 1
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Run Test Case 1
         */
        String result = deploymentService.getFcState();
        assertNotNull("Result of getFcState must not be null", result);
        assertEquals("Wrong invokation result", result,
                org.objectweb.fractal.api.control.LifeCycleController.STOPPED);
    }

    public void testGetServiceAssemblyDescriptor() {
        /*
         * Test Case 1 The parameter ServiceAssemblyName is null, an exception
         * is raised
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        try {
            method1 = DeploymentService.class.getDeclaredMethod(
                    "getJbiDescriptorAsString", new Class[] {URI.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[] {method1};
        DeploymentService deploymentService = createMock(
                DeploymentService.class, mockedMethods);
        /*
         * Create Test parameters
         */
        String serviceAssemblyName = null;
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.getServiceAssemblyDescriptor(serviceAssemblyName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    " must not be null"));
        }
        /*
         * Test Case 2 The parameter ServiceAssemblyName is empty, an exception
         * is raised
         */
        /*
         * Create Test parameters
         */
        serviceAssemblyName = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.getServiceAssemblyDescriptor(serviceAssemblyName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }

        /*
         * Test Case 3
         */
        /*
         * Create Test parameters
         */
        LifeCycleManagerImpl lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        serviceAssemblyName = "service assembly";
        ServiceAssemblyLifeCycle saLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        URI installationURI = null;
        try {
            installationURI = new URI("file://installation");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create test parameters");
        }
        org.objectweb.petals.util.monolog.wrapper.javalog.Logger logger = createMock(org.objectweb.petals.util.monolog.wrapper.javalog.Logger.class);
        deploymentService.logger = logger;
        String resultDescriptor = "result";
        String result = null;
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyByName(serviceAssemblyName))
                .andReturn(saLifeCycle);
        expect(saLifeCycle.getInstallationRoot()).andReturn(installationURI);
        expect(deploymentService.getJbiDescriptorAsString(isA(URI.class)))
                .andReturn(resultDescriptor);
        replay(lifeCycleManager);
        replay(saLifeCycle);
        replay(deploymentService);
        deploymentService.managerService = lifeCycleManager;
        /*
         * Run Test Case 3
         */
        try {
            result = deploymentService
                    .getServiceAssemblyDescriptor(serviceAssemblyName);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        verify(saLifeCycle);
        verify(lifeCycleManager);
        verify(deploymentService);
        assertNotNull(
                "Result of getServiceAssemblyDescriptor must be non null",
                result);
        assertEquals("Wrong invokation result", result, resultDescriptor);
    }
    
    public void testInit() {
        /*
         * Test Case 1
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        /*
         * Run Test Case 1
         */
        deploymentService.init();
        assertNotNull("Initialization incomplete",deploymentService.log);
        assertNotNull("Initialization incomplete",deploymentService.packageHandler);
    }

    public void testIsDeployedServiceUnit() {
        /*
         * Test Case 1 The parameter serviceUnitName is null, an exception is
         * raised
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         * 
         */
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        String serviceUnitName = null;
        String componentName = null;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.isDeployedServiceUnit(componentName,
                    serviceUnitName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }

        /*
         * Test Case 2 The parameter serviceUnitName is empty, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        serviceUnitName = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.isDeployedServiceUnit(componentName,
                    serviceUnitName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }

        /*
         * Test Case 3 The parameter componentName is null, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        serviceUnitName = "service unit name";
        /*
         * Run Test Case 3
         */
        try {
            deploymentService.isDeployedServiceUnit(componentName,
                    serviceUnitName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }

        /*
         * Test Case 4 The parameter componentName is empty, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        componentName = "";
        /*
         * Run Test Case 4
         */
        try {
            deploymentService.isDeployedServiceUnit(componentName,
                    serviceUnitName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }

        /*
         * Test Case 5 There is a service unit with the given name deployed for
         * this component
         */
        /*
         * Create Test parameters
         */
        componentName = "fooComponent";
        String saName = "service assembly name";
        LifeCycleManagerImpl lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        Map<String, ServiceAssemblyLifeCycle> serviceAssemblies = new HashMap<String, ServiceAssemblyLifeCycle>();
        ServiceAssemblyLifeCycle saLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        serviceAssemblies.put(saName, saLifeCycle);
        ServiceAssembly sa = createMock(ServiceAssembly.class);
        List<ServiceUnit> sus = new ArrayList<ServiceUnit>();
        ServiceUnit su = createMock(ServiceUnit.class);
        sus.add(su);
        Identification identification = createMock(Identification.class);
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyLifeCycles()).andReturn(
                serviceAssemblies);
        expect(saLifeCycle.getServiceAssembly()).andReturn(sa);
        expect(sa.getServiceUnits()).andReturn(sus);
        expect(identification.getName()).andReturn(serviceUnitName);
        expect(su.getTargetComponentName()).andReturn(componentName);
        expect(su.getIdentification()).andReturn(identification);
        replay(sa);
        replay(su);
        replay(lifeCycleManager);
        replay(saLifeCycle);
        replay(identification);
        deploymentService.managerService = lifeCycleManager;
        boolean result = false;
        /*
         * Run Test Case 5
         */
        try {
            result = deploymentService.isDeployedServiceUnit(componentName,
                    serviceUnitName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertTrue("Wrong invokation result", result);
        verify(sa);
        verify(su);
        verify(lifeCycleManager);
        verify(saLifeCycle);
        verify(identification);
    }
    
    public void testListFc() {
        /*
         * Test Case 1
         */
        DeploymentService deploymentService = new DeploymentService();
        String[] result = null;
        /*
         * Run Test Case 1
         */
        result = deploymentService.listFc();
        assertNotNull("The result of listFc must not be null", result);
        assertEquals("Wrong invokation result",
                DeploymentService.CLIENT_INTERFACES, result);
    }

    public void testLookupFc() {
        /*
         * Test Case 1 The requested interface does not belong to adminService
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create test parameters
         */
        String clientItf = "foo";
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.lookupFc(clientItf);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The requested interface belongs to adminService
         */
        /*
         * Create test parameters
         */
        clientItf = InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME;
        Object result = null;
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        deploymentService.managerService = lifeCycleManager;
        /*
         * Run Test Case 2
         */
        try {
            result = deploymentService.lookupFc(clientItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Exception during method invokation");
        }
        assertEquals("Wrong result for invokation", result, lifeCycleManager);

        /*
         * Test Case 3 The requested interface belongs to adminService
         */
        /*
         * Create test parameters
         */
        clientItf = InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME;
        RepositoryService repoService = createMock(RepositoryImpl.class);
        deploymentService.repositorySrv = repoService;
        /*
         * Run Test Case 3
         */
        try {
            result = deploymentService.lookupFc(clientItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Exception during method invokation");
        }
        assertEquals("Wrong result for invokation", result, repoService);

        /*
         * Test Case 4 The requested interface belongs to adminService
         */
        /*
         * Create test parameters
         */
        clientItf = InterfaceNamesHelper.JBI_ADMIN_NAME;
        AdminServiceMBean adminService = createMock(AdminService.class);
        deploymentService.adminSrv = adminService;
        /*
         * Run Test Case 4
         */
        try {
            result = deploymentService.lookupFc(clientItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Exception during method invokation");
        }
        assertEquals("Wrong result for invokation", result, adminService);

        /*
         * Test Case 5 The requested interface belongs to adminService
         */
        /*
         * Create test parameters
         */
        clientItf = InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME;
        SystemState sysState = createMock(SystemStateImpl.class);
        deploymentService.recoverySrv = sysState;
        /*
         * Run Test Case 5
         */
        try {
            result = deploymentService.lookupFc(clientItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Exception during method invokation");
        }
        assertEquals("Wrong result for invokation", result, sysState);

        /*
         * Test Case 6 The requested interface belongs to adminService
         */
        /*
         * Create test parameters
         */
        clientItf = InterfaceNamesHelper.JBI_ENDPOINTSERVICE_NAME;
        EndpointService epService = createMock(EndpointServiceImpl.class);
        deploymentService.endpointService = epService;
        /*
         * Run Test Case 6
         */
        try {
            result = deploymentService.lookupFc(clientItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Exception during method invokation");
        }
        assertEquals("Wrong result for invokation", result, epService);
    }
    
    public void testShutdown() {
        /*
         * Test Case 1 serviceAssemblyName parameter is null, an exception is
         * raised
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        String serviceAssemblyName = null;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.shutDown(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }
        /*
         * Test Case 2 serviceAssemblyName parameter is empty, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        serviceAssemblyName = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.shutDown(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }
    }
    
    public void testShutdown2() {
        /*
         * Test Case 1 
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;

        Logger logger = createMock(LoggerImpl.class);
        deploymentService.logger = logger;
        /*
         * Run Test Case 1
         */
        deploymentService.shutdown();
        assertNull("ShutDown action was not properly realized",deploymentService.logger);
    }

    public void testStart() {
        /*
         * Test Case 1 serviceAssemblyName parameter is null, an exception is
         * raised
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        String serviceAssemblyName = null;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.start(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }
        /*
         * Test Case 2 serviceAssemblyName parameter is empty, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        serviceAssemblyName = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.start(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }
    }

    public void testStartFc() {
        /*
         * Test Case 1
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        try {
            method1 = DeploymentService.class.getDeclaredMethod("init",
                    new Class[] {});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[] {method1};
        DeploymentService deploymentService = createMock(
                DeploymentService.class, mockedMethods);
        deploymentService.init();
        replay(deploymentService);
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.startFc();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertEquals("DeploymentService was not properly initialized",
                deploymentService.fcState,
                org.objectweb.fractal.api.control.LifeCycleController.STARTED);
        verify(deploymentService);
    }

    public void testStop() {
        /*
         * Test Case 1 serviceAssemblyName parameter is null, an exception is
         * raised
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        String serviceAssemblyName = null;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.stop(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }
        /*
         * Test Case 2 serviceAssemblyName parameter is empty, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        serviceAssemblyName = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.stop(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }
    }

    public void testStopFc() {
        /*
         * Test Case 1
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        try {
            method1 = DeploymentService.class.getDeclaredMethod("shutdown",
                    new Class[] {});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[] {method1};
        DeploymentService deploymentService = createMock(
                DeploymentService.class, mockedMethods);
        deploymentService.shutdown();
        replay(deploymentService);
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.stopFc();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertEquals("DeploymentService was not properly initialized",
                deploymentService.fcState,
                org.objectweb.fractal.api.control.LifeCycleController.STOPPED);
        verify(deploymentService);
    }
    
    public void testUnbindFc() {
        /*
         * Test Case 1 The parameter clientItfName is null, an exception will be
         * raised
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        String clientItfName = null;
        Object serverItf = null;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.unbindFc(clientItfName);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The parameter clientItfName match the logger interface
         * name The logger attribute of deploymentService will be set to null
         */
        serverItf = createMock(LoggerImpl.class);
        deploymentService.logger = (Logger) serverItf;
        clientItfName = InterfaceNamesHelper.LOGGER_NAME;
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("logger attribute was not properly set",
                deploymentService.logger, null);

        /*
         * Test Case 3 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME The managerService
         * attribute of deploymentService will be set to null
         */
        serverItf = createMock(LifeCycleManagerImpl.class);
        deploymentService.managerService = (LifeCycleManagerService) serverItf;
        clientItfName = InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME;
        /*
         * Run Test Case 3
         */
        try {
            deploymentService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                deploymentService.managerService, null);

        /*
         * Test Case 4 The parameter clientItfName match the
         * InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME The repositorySrv
         * attribute of deploymentService will be set to null
         */
        serverItf = createMock(RepositoryImpl.class);
        clientItfName = InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME;
        deploymentService.repositorySrv = (RepositoryService) serverItf;
        /*
         * Run Test Case 4
         */
        try {
            deploymentService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                deploymentService.repositorySrv, null);

        /*
         * Test Case 5 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_ADMIN_NAMEThe adminSrv attribute of
         * deploymentService will be set to null
         */
        serverItf = createMock(AdminService.class);
        clientItfName = InterfaceNamesHelper.JBI_ADMIN_NAME;
        deploymentService.adminSrv = (javax.jbi.management.AdminServiceMBean) serverItf;
        /*
         * Run Test Case 5
         */
        try {
            deploymentService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                deploymentService.adminSrv, null);

        /*
         * Test Case 6 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME The recoverySrv attribute
         * of deploymentService will be set to null
         */
        serverItf = createMock(SystemStateImpl.class);
        clientItfName = InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME;
        deploymentService.recoverySrv = (SystemState) serverItf;
        /*
         * Run Test Case 6
         */
        try {
            deploymentService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                deploymentService.recoverySrv, null);

        /*
         * Test Case 7 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_ENDPOINTSERVICE_NAME The recoverySrv
         * attribute of endpointService will be set to serverItf value
         */
        serverItf = createMock(EndpointServiceImpl.class);
        clientItfName = InterfaceNamesHelper.JBI_ENDPOINTSERVICE_NAME;
        deploymentService.endpointService = (EndpointService) serverItf;
        /*
         * Run Test Case 7
         */
        try {
            deploymentService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                deploymentService.endpointService, null);
    }

    public void testUndeploy() {

    }

    @Override
    protected void setUp() {

    }
}
