/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JoramTransporterTest.java 1:42:16 PM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.transport;

import java.lang.reflect.Method;

import javax.jbi.JBIException;
import javax.jms.JMSException;
import javax.jms.ObjectMessage;
import javax.jms.TextMessage;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.joram.client.jms.Message;
import org.objectweb.joram.client.jms.Queue;
import org.objectweb.joram.client.jms.QueueConnection;
import org.objectweb.joram.client.jms.QueueConnectionFactory;
import org.objectweb.joram.client.jms.QueueSender;
import org.objectweb.joram.client.jms.QueueSession;
import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;
import org.objectweb.petals.jbi.registry.ConsumerEndpoint;
import org.objectweb.petals.jbi.routing.RouterImpl;
import org.objectweb.petals.jbi.routing.RoutingException;
import org.objectweb.petals.jbi.transport.mock.MockInitialContext;
import org.objectweb.petals.jbi.transport.mock.MockJoramQueue;
import org.objectweb.petals.jbi.transport.mock.MockRouter;
import org.objectweb.petals.jbi.transport.mock.MockSerializer;
import org.objectweb.petals.util.InterfaceNamesHelper;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.Monolog;

/**
 * test of the JoramTransporter
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class JoramTransporterTest extends TestCase {

    protected JoramTransporter transporter;

    public void setUp() {
        transporter = new JoramTransporter();
    }

    public void testBindFcException() {
        try {
            transporter.bindFc("foo", null);
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testBindFcLookupFcUnbindFc() throws NoSuchInterfaceException,
        IllegalBindingException, IllegalLifeCycleException {
        transporter.bindFc(InterfaceNamesHelper.LOGGER_NAME, Monolog
            .initialize().getLogger("test"));
        assertNotNull(transporter.log);
        assertNotNull(transporter.lookupFc(InterfaceNamesHelper.LOGGER_NAME));
        transporter.unbindFc(InterfaceNamesHelper.LOGGER_NAME);
        assertNull(transporter.log);

        RouterImpl routerImpl = new RouterImpl();
        transporter.bindFc(InterfaceNamesHelper.JBI_ROUTER_NAME, routerImpl);
        assertEquals(transporter.router, routerImpl);
        assertEquals(routerImpl, transporter
            .lookupFc(InterfaceNamesHelper.JBI_ROUTER_NAME));
        transporter.unbindFc(InterfaceNamesHelper.JBI_ROUTER_NAME);
        assertNull(transporter.router);
    }

    public void testGetFcState() {
        transporter.fcState = LifeCycleController.STOPPED;
        assertEquals(LifeCycleController.STOPPED, transporter.getFcState());
        transporter.fcState = LifeCycleController.STARTED;
        assertEquals(LifeCycleController.STARTED, transporter.getFcState());
    }

    public void testListFc() {
        assertEquals(transporter.listFc()[0],
            InterfaceNamesHelper.JBI_ROUTER_NAME);
    }

    public void testLookupFcException() {
        try {
            transporter.lookupFc("foo");
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testSendDistant() throws NamingException, JMSException,
        JBIException, SecurityException, NoSuchMethodException {
        MessageExchangeImpl messageExchangeImpl = new MessageExchangeImpl(
            new ConsumerEndpoint("compo", "0"));
        transporter.log = EasyMock.createMock(LoggingUtil.class);
        MockInitialContext initialContext = new MockInitialContext();
        Queue queue = EasyMock.createMock(Queue.class);

        initialContext.getContext().getBindings().put("JoramQueue-0", queue);
        transporter.context = initialContext;

        JoramQueue joramQueue = EasyMock.createMock(JoramQueue.class);
        QueueConnectionFactory queueConnectionFactory = EasyMock
            .createMock(QueueConnectionFactory.class);
        Message message = EasyMock.createMock(Message.class);
        QueueSession queueSession = EasyMock.createMock(QueueSession.class);
        QueueConnection queueConnection = EasyMock
            .createMock(QueueConnection.class);
        QueueSender queueSender = EasyMock.createMock(QueueSender.class);

        EasyMock.expect(
            queueConnectionFactory.createQueueConnection("petals", "petals"))
            .andReturn(queueConnection).anyTimes();
        EasyMock.replay(queueConnectionFactory);
        EasyMock.expect(joramQueue.getQueueConnectionFactory()).andReturn(
            queueConnectionFactory).anyTimes();
        EasyMock.expect(queueConnection.createQueueSession(true, 0)).andReturn(
            queueSession).anyTimes();
        EasyMock.expect(queueSession.createSender(queue))
            .andReturn(queueSender).anyTimes();

        queueSession.commit();
        queueConnection.close();

        EasyMock.replay(queueConnection);
        EasyMock.replay(queueSession);
        EasyMock.replay(queue);
        EasyMock.replay(joramQueue);
        EasyMock.replay(message);

        transporter.joramQueue = joramQueue;
        transporter.serializer = new MockSerializer();

        transporter.user = "petals";
        transporter.pwd = "petals";

        transporter.sendDistant(messageExchangeImpl, "0");
    }

    public void testSendDistanException() throws NamingException, JMSException,
        JBIException, SecurityException, NoSuchMethodException {
        MessageExchangeImpl messageExchangeImpl = new MessageExchangeImpl(
            new ConsumerEndpoint("compo", "0"));
        transporter.log = EasyMock.createMock(LoggingUtil.class);
        MockInitialContext initialContext = new MockInitialContext();
        Queue queue = EasyMock.createMock(Queue.class);

        initialContext.getContext().getBindings().put("JoramQueue-0", queue);
        transporter.context = initialContext;

        JoramQueue joramQueue = EasyMock.createMock(JoramQueue.class);
        QueueConnectionFactory queueConnectionFactory = EasyMock
            .createMock(QueueConnectionFactory.class);
        Message message = EasyMock.createMock(Message.class);
        QueueSession queueSession = EasyMock.createMock(QueueSession.class);
        QueueConnection queueConnection = EasyMock
            .createMock(QueueConnection.class);

        EasyMock.expect(joramQueue.getQueueConnectionFactory()).andReturn(
            queueConnectionFactory).anyTimes();
        EasyMock.expect(
            queueConnectionFactory.createQueueConnection("petals", "petals"))
            .andThrow(new RuntimeException("error"));

        queueSession.commit();
        queueConnection.close();

        EasyMock.replay(queueConnection);
        EasyMock.replay(queueSession);
        EasyMock.replay(queue);
        EasyMock.replay(joramQueue);
        EasyMock.replay(message);

        transporter.joramQueue = joramQueue;
        transporter.serializer = new MockSerializer();

        transporter.user = "petals";
        transporter.pwd = "petals";
        try {
            transporter.sendDistant(messageExchangeImpl, "0");
            fail();
        } catch (TransportException e) {
            // do nothing
        }
    }

    public void testSend() throws SecurityException, NoSuchMethodException,
        TransportException {
        transporter = EasyMock.createMock(JoramTransporter.class,
            new Method[] {JoramTransporter.class.getDeclaredMethod(
                "sendDistant", new Class[] {MessageExchangeImpl.class,
                    String.class})});

        transporter.sendDistant(null, "0");

        EasyMock.replay(transporter);

        transporter.send(null, "0");
    }

    public void testStartFc() throws Exception {
        transporter = EasyMock
            .createMock(JoramTransporter.class, new Method[] {
                JoramTransporter.class.getDeclaredMethod("initJMS",
                    new Class[0]),
                JoramTransporter.class.getDeclaredMethod("startJMS",
                    new Class[0])});

        transporter.initJMS();
        transporter.startJMS();

        EasyMock.replay(transporter);

        transporter.startFc();
        assertEquals(LifeCycleController.STARTED, transporter.getFcState());
    }

    public void testUnbindFcException() {
        try {
            transporter.unbindFc("foo");
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testStartJMS() throws JBIException {
        MockJoramQueue joramQueue = new MockJoramQueue();

        transporter.log = EasyMock.createMock(LoggingUtil.class);
        transporter.joramQueue = joramQueue;

        transporter.startJMS();
        assertTrue(joramQueue.isStartListening());
    }

    public void testStartJMSException() throws JBIException {
        MockJoramQueue joramQueue = new MockJoramQueue();
        joramQueue.setThrowException(true);

        transporter.log = EasyMock.createMock(LoggingUtil.class);
        transporter.joramQueue = joramQueue;
        try {
            transporter.startJMS();
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testStopFc() throws IllegalLifeCycleException {
        transporter.fcState = LifeCycleController.STARTED;
        transporter.stopFc();
        assertEquals(transporter.fcState, LifeCycleController.STOPPED);
    }

    public void testStopJMS() throws JBIException {
        transporter.log = EasyMock.createMock(LoggingUtil.class);

        MockJoramQueue joramQueue = new MockJoramQueue();

        transporter.joramQueue = joramQueue;
        transporter.context = EasyMock.createMock(InitialContext.class);

        transporter.stopJMS();

        assertTrue(joramQueue.isUnbindQueue());
        assertTrue(joramQueue.isStopListening());
    }

    public void testSendLocal() throws TransportException, RoutingException {
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);
        RouterImpl routerImpl = EasyMock.createMock(RouterImpl.class);

        routerImpl.receive(messageExchangeImpl);

        EasyMock.replay(routerImpl);

        transporter.log = EasyMock.createMock(LoggingUtil.class);
        transporter.router = routerImpl;

        transporter.sendLocal(messageExchangeImpl);
    }

    public void testSendLocalException() {
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);

        transporter.log = EasyMock.createMock(LoggingUtil.class);
        transporter.router = new MockRouter();

        try {
            transporter.sendLocal(messageExchangeImpl);
            fail();
        } catch (TransportException e) {
            // do nothing
        }
    }

    public void testInitJMS() throws Exception {
        transporter = EasyMock.createMock(JoramTransporter.class,
            new Method[] {JoramTransporter.class.getDeclaredMethod(
                "createJoramQueue", new Class[0])});
        JoramQueue joramQueue = EasyMock.createMock(JoramQueue.class);

        EasyMock.expect(transporter.createJoramQueue()).andReturn(joramQueue)
            .anyTimes();
        joramQueue.init();

        EasyMock.replay(joramQueue);
        EasyMock.replay(transporter);

        transporter.log = EasyMock.createMock(LoggingUtil.class);

        transporter.initJMS();

        assertNotNull(transporter.host);
        assertEquals(transporter.user, "petals");
        assertEquals(transporter.pwd, "petals");
    }

    public void testProcessIncomingMessage() throws JBIException {
        ObjectMessage objectMessage = EasyMock.createMock(ObjectMessage.class);
        Serializer serializer = EasyMock.createMock(Serializer.class);
        RouterImpl routerImpl = EasyMock.createMock(RouterImpl.class);
        MessageExchangeImpl messageExchangeImpl = new MessageExchangeImpl(
            new ConsumerEndpoint("compo", "0"));

        EasyMock.expect(serializer.jms2jbi(objectMessage)).andReturn(
            messageExchangeImpl).anyTimes();
        routerImpl.receive(messageExchangeImpl);

        EasyMock.replay(routerImpl);
        EasyMock.replay(objectMessage);
        EasyMock.replay(serializer);

        transporter.serializer = serializer;
        transporter.router = routerImpl;
        transporter.log = EasyMock.createMock(LoggingUtil.class);

        transporter.processIncomingMessage(objectMessage);
    }

    public void testProcessIncomingMessageException() throws JBIException {
        TextMessage textMessage = EasyMock.createMock(TextMessage.class);

        transporter.log = EasyMock.createMock(LoggingUtil.class);

        transporter.processIncomingMessage(textMessage);
    }
}
