/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SharedLibrariesClassLoaderTest.java 17:06:54 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.classloader;

import java.io.DataInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

/**
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class ComponentClassLoaderTest extends TestCase {

    /**
     * ComponentClassLoader to test
     */
    private ComponentClassLoader componentClassLoader;

    /**
     * Shared library class loader
     */
    private SharedLibraryClassLoader sharedLibraryClassLoader;

    public void setUp() throws IOException {
        SharedLibrariesClassLoader sharedLibrariesClassLoader = new SharedLibrariesClassLoader();
        List<URL> baseUrls = new ArrayList<URL>();
        List<String> classpathLocations = new ArrayList<String>();
        classpathLocations.add("petals-core-test.jar");
        String baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        baseUrls.add(new URL("file:" + baseDir.replace(File.separator, "/")
            + "src/test-data/"));
        baseUrls.add(new URL("file:" + baseDir.replace(File.separator, "/")
            + "target/test-classes/"));
        baseUrls.add(new URL("file:" + baseDir.replace(File.separator, "/")
            + "target/"));
        sharedLibraryClassLoader = new SharedLibraryClassLoader(baseUrls
                .toArray(new URL[0]), classpathLocations);
        sharedLibraryClassLoader.setParentFirst(false);
        sharedLibrariesClassLoader.addSharedLibraryClassLoader("1",
                sharedLibraryClassLoader);

        baseUrls = new ArrayList<URL>();
        classpathLocations = new ArrayList<String>();
        classpathLocations.add("petals-common-test.jar");

        baseDir = baseDir.replace("core", "container");
        baseUrls.add(new URL("file:" + baseDir.replace(File.separator, "/")
            + "src/test-data/"));
        // baseUrls.add(new URL("file:" + baseDir.replace(File.separator, "/") +
        // "target/classes/"));
        List<String> slNames = new ArrayList<String>();
        slNames.add("1");
        componentClassLoader = new ComponentClassLoader(baseUrls
                .toArray(new URL[0]), classpathLocations,
                sharedLibrariesClassLoader, slNames);
    }

    /**
     * Display the classpath
     */
    public void testGetClasspath() {
        // System.out.println("testGetClasspath - display the classpaths");
        String compoCP = "testGetClasspath - component classpath "
            + componentClassLoader.getClasspath();
        compoCP.length();
        // System.out.println(compoCP);
        String slCP = "testGetClasspath - shared library classpath "
            + sharedLibraryClassLoader.getClasspath();
        slCP.length();
        // System.out.println(slCP);
    }

    /**
     * Test to get a resource from itself The resource is in the component
     * classpath and in the shared library classpath
     */
    public void testGetResourceInSelfAndParentSelfFirst() {
        // System.out.println("testGetResourceInSelfAndParentSelfFirst - try to
        // get a resource from itself (the resource is in our and in the parent
        // classpath)");
        componentClassLoader.setParentFirst(false);
        URL url = componentClassLoader.getResource("resourceBoth.txt");
        if (url != null) {
            // System.out.println("testGetResourceInSelfAndParentSelfFirst -
            // resource found " + url.getPath());
        } else {
            // System.out.println("testGetResourceInSelfAndParentSelfFirst -
            // resource not found ");
            fail("testGetResourceInSelfAndParentSelfFirst - resource not found ");
        }
    }

    /**
     * Test to get a resource from its parent The resource is in the component
     * classpath and in the shared library classpath
     */
    public void testGetResourceInSelfAndParentParentFirst() {
        // System.out.println("testGetResourceInSelfAndParentParentFirst - try
        // to get a resource from its parent (the resource is in our and in the
        // parent classpath)");
        componentClassLoader.setParentFirst(true);
        URL url = componentClassLoader.getResource("resourceBoth.txt");
        if (url != null) {
            // System.out.println("testGetResourceInSelfAndParentParentFirst -
            // resource found " + url.getPath());
        } else {
            // System.out.println("testGetResourceInSelfAndParentParentFirst -
            // resource not found ");
            fail("testGetResourceInSelfAndParentParentFirst - resource not found ");
        }
    }

    /**
     * Test to get a resource as stream from itself The resource is in the
     * component classpath and in the shared library classpath
     * 
     * @throws IOException
     */
    public void testGetResourceAsStreamInSelfAndParentSelfFirst()
        throws IOException {
        // System.out.println("testGetResourceAsStreamInSelfAndParentSelfFirst -
        // try to get a resource from itself (the resource is in our and in the
        // parent classpath)");
        componentClassLoader.setParentFirst(false);
        InputStream inputStream = componentClassLoader
                .getResourceAsStream("resourceBoth.txt");
        if (inputStream != null) {
            DataInputStream dis = new DataInputStream(inputStream);
            byte[] content = new byte[dis.available()];
            dis.readFully(content);
            // System.out.println("testGetResourceAsStreamInSelfAndParentSelfFirst
            // - content of the stream\n" + new String(content));
        } else {
            // System.out.println("testGetResourceAsStreamInSelfAndParentSelfFirst
            // - resource not found ");
            fail("testGetResourceAsStreamInSelfAndParentSelfFirst - resource not found ");
        }
    }

    /**
     * Test to get a resource as stream from its parent The resource is in the
     * component classpath and in the shared library classpath
     * 
     * @throws IOException
     * 
     */
    public void testGetResourceAsStreamInSelfAndParentParentFirst()
        throws IOException {
        // System.out.println("testGetResourceAsStreamInSelfAndParentParentFirst
        // - try to get a resource from its parent (the resource is in our and
        // in the parent classpath)");
        componentClassLoader.setParentFirst(true);
        InputStream inputStream = componentClassLoader
                .getResourceAsStream("resourceBoth.txt");
        if (inputStream != null) {
            DataInputStream dis = new DataInputStream(inputStream);
            byte[] content = new byte[dis.available()];
            dis.readFully(content);
            // System.out.println("testGetResourceAsStreamInSelfAndParentParentFirst
            // - content of the stream\n" + new String(content));
        } else {
            // System.out.println("testGetResourceAsStreamInSelfAndParentParentFirst
            // - resource not found ");
            fail("testGetResourceAsStreamInSelfAndParentParentFirst - resource not found ");
        }
    }

    /**
     * Test to get a resource as stream from its self The resource is in the
     * shared library classpath
     * 
     * @throws IOException
     */
    public void testGetResourceAsStreamInParentSelfFirst() throws IOException {
        // System.out.println("testGetResourceAsStreamInParentSelfFirst - try to
        // get a resource from its parent using our own classpath first");
        componentClassLoader.setParentFirst(false);
        InputStream inputStream = componentClassLoader
                .getResourceAsStream("resourceCore.txt");
        if (inputStream != null) {
            DataInputStream dis = new DataInputStream(inputStream);
            byte[] content = new byte[dis.available()];
            dis.readFully(content);
            // System.out.println("testGetResourceAsStreamInParentSelfFirst -
            // content of the stream\n" + new String(content));
        } else {
            // System.out.println("testGetResourceAsStreamInParentSelfFirst -
            // resource not found ");
            fail("testGetResourceAsStreamInParentSelfFirst - resource not found ");
        }
    }

    /**
     * Test to get a resource as stream from its parent The resource is in the
     * component classpath
     * 
     * @throws IOException
     */
    public void testGetResourceAsStreamInSelfParentFirst() throws IOException {
        // System.out.println("testGetResourceAsStreamInSelfParentFirst - try to
        // get a resource from its self using the parent classpath first");
        componentClassLoader.setParentFirst(true);
        InputStream inputStream = componentClassLoader
                .getResourceAsStream("resourceContainer.txt");
        if (inputStream != null) {
            DataInputStream dis = new DataInputStream(inputStream);
            byte[] content = new byte[dis.available()];
            dis.readFully(content);
            // System.out.println("testGetResourceAsStreamInSelfParentFirst -
            // content of the stream\n" + new String(content));
        } else {
            // System.out.println("testGetResourceAsStreamInSelfParentFirst -
            // resource not found ");
            fail("testGetResourceAsStreamInSelfParentFirst - resource not found ");
        }
    }

    /**
     * Test to load a class from its parent
     */
    public void testLoadClassParentFirst() {
        // System.out.println("testLoadClassParentFirst - try to load a class
        // from its parent");
        componentClassLoader.setParentFirst(true);
        Class clazz;
        try {
            clazz = componentClassLoader.loadClass(
                    "org.objectweb.petals.ServiceContext", false);
            if (clazz != null) {
                // System.out.println("testLoadClassParentFirst - class loaded "
                // + clazz.getName());
            } else {
                // System.out.println("testLoadClassParentFirst - class not
                // found ");
                fail("testLoadClassParentFirst - class not found ");
            }
        } catch (ClassNotFoundException e) {
            // System.out.println("testLoadClassParentFirst - class not found
            // ");
            fail("testLoadClassParentFirst - class not found ");
        }
    }

    /**
     * Test to load a class from its self
     */
    public void testLoadClassSelfFirst() {
        // System.out.println("testLoadClassSelfFirst - try to load a class from
        // its self");
        componentClassLoader.setParentFirst(false);
        Class clazz;
        try {
            clazz = componentClassLoader.loadClass(
                    "org.objectweb.petals.PetalsException", false);
            if (clazz != null) {
                // System.out.println("testLoadClassSelfFirst - class loaded " +
                // clazz.getName());
            } else {
                // System.out.println("testLoadClassSelfFirst - class not found
                // ");
                fail("testLoadClassSelfFirst - class not found ");
            }
        } catch (ClassNotFoundException e) {
            // System.out.println("testLoadClassSelfFirst - class not found ");
            fail("testLoadClassSelfFirst - class not found ");
        }
    }

    /**
     * Test to load a class from its parent The class is in both classpath
     * (parent and self)
     * 
     * @throws IllegalAccessException
     * @throws InstantiationException
     */
    public void testLoadClassInSelfAndParentParentFirst()
        throws InstantiationException, IllegalAccessException {
        // System.out.println("testLoadClassInSelfAndParentParentFirst - try to
        // load a class from its parent (the class is in both classpath)");
        componentClassLoader.setParentFirst(true);
        Class clazz;
        try {
            clazz = componentClassLoader.loadClass(
                    "org.objectweb.petals.classloader.TestClassloader", false);
            if (clazz != null) {
                // The new instance will enable us to verify which class we got
                // clazz.newInstance();
                // System.out.println("testLoadClassInSelfAndParentParentFirst -
                // class loaded " + clazz.getName());
            } else {
                // System.out.println("testLoadClassInSelfAndParentParentFirst -
                // class not found ");
                fail("testLoadClassInSelfAndParentParentFirst - class not found ");
            }
        } catch (ClassNotFoundException e) {
            // System.out.println("testLoadClassInSelfAndParentParentFirst -
            // class not found ");
            fail("testLoadClassInSelfAndParentParentFirst - class not found ");
        }
    }

    /**
     * Test to load a class from its self The class is in both classpath (parent
     * and self)
     * 
     * @throws IllegalAccessException
     * @throws InstantiationException
     */
    public void testLoadClassInSelfAndParentSelfFirst()
        throws InstantiationException, IllegalAccessException {
        // System.out.println("testLoadClassInSelfAndParentSelfFirst - try to
        // load a class from its self (the class is in both classpath)");
        componentClassLoader.setParentFirst(false);
        Class clazz;
        try {
            clazz = componentClassLoader.loadClass(
                    "org.objectweb.petals.classloader.TestClassloader", false);
            if (clazz != null) {
                // The new instance will enable us to verify which class we got
                // clazz.newInstance();
                // System.out.println("testLoadClassInSelfAndParentSelfFirst -
                // class loaded " + clazz.getName());
            } else {
                // System.out.println("testLoadClassInSelfAndParentSelfFirst -
                // class not found ");
                fail("testLoadClassInSelfAndParentSelfFirst - class not found ");
            }
        } catch (ClassNotFoundException e) {
            // System.out.println("testLoadClassInSelfAndParentSelfFirst - class
            // not found ");
            fail("testLoadClassInSelfAndParentSelfFirst - class not found ");
        }
    }

}
